-- ********************
-- RECTIFICACIO:
-- Taula de COEFICIENTS
-- ********************


--
-- ZONES
--

CREATE TABLE ZONES
(
  CODI VARCHAR2(3) CONSTRAINT ZONES_CODI_NN NOT NULL 
, NOM VARCHAR2(50) CONSTRAINT ZONES_NOM_NN NOT NULL 
, CONSTRAINT ZONES_PK PRIMARY KEY(CODI)
);

--
-- COMPETICIONS
--

CREATE TABLE COMPETICIONS
(
  CODI VARCHAR2(20) CONSTRAINT COMPETICIONS_CODI_NN NOT NULL 
, DESCRIPCIO VARCHAR2(50) CONSTRAINT COMPETICIONS_DESCRIPCIO_NN NOT NULL 
, ZONA VARCHAR2(3) CONSTRAINT COMPETICIONS_ZONA_NN NOT NULL 
, CONSTRAINT COMPETICIONS_PK PRIMARY KEY(CODI)
);

ALTER TABLE COMPETICIONS 
ADD CONSTRAINT COMPETICIONS_ZONA_FK 
FOREIGN KEY(ZONA) 
REFERENCES ZONES(CODI);

--
-- TEMPORADES
--

CREATE TABLE TEMPORADES
(
  COMPETICIO VARCHAR2(20) CONSTRAINT TEMPORADES_COMPETICIO_NN NOT NULL 
, AAAA NUMBER CONSTRAINT TEMPORADES_AAAA_NN NOT NULL 
, INICI DATE 
, FINAL DATE 
, CONSTRAINT TEMPORADES_PK PRIMARY KEY(COMPETICIO, AAAA)
);

ALTER TABLE TEMPORADES 
ADD CONSTRAINT TEMPORADES_COMPETICIO_FK 
FOREIGN KEY(COMPETICIO) 
REFERENCES COMPETICIONS(CODI);

--
-- EQUIPS
--

CREATE TABLE EQUIPS
(
  CODI VARCHAR2(3) CONSTRAINT EQUIPS_CODI_NN NOT NULL 
, NOM VARCHAR(50) CONSTRAINT EQUIPS_NOM_NN NOT NULL 
, FUNDACIO DATE 
, PAIS VARCHAR2(3) CONSTRAINT EQUIPS_PAIS_NN NOT NULL
, CONSTRAINT EQUIPS_PK PRIMARY KEY(CODI)
);

ALTER TABLE EQUIPS 
ADD CONSTRAINT EQUIPS_PAIS_FK 
FOREIGN KEY(PAIS) 
REFERENCES ZONES(CODI);

--
-- PARTITS
--

CREATE TABLE PARTITS
(
  ID NUMBER CONSTRAINT PARTITS_ID_NN NOT NULL
, COMPETICIO VARCHAR2(20) CONSTRAINT PARTITS_COMPETICIO_NN NOT NULL 
, AAAA NUMBER CONSTRAINT PARTITS_AAAA_NN NOT NULL 
, EQUIP1 VARCHAR2(3) CONSTRAINT PARTITS_EQUIP1_NN NOT NULL 
, EQUIP2 VARCHAR2(3) CONSTRAINT PARTITS_EQUIP2_NN NOT NULL 
, DATA DATE CONSTRAINT PARTITS_DATA_NN NOT NULL 
, JUGAT VARCHAR2(1) DEFAULT '0' CONSTRAINT PARTITS_JUGAT_NN NOT NULL 
, CONSTRAINT PARTITS_PK PRIMARY KEY(ID)
, CONSTRAINT PARTIT_JUGAT_CK CHECK (JUGAT IN (0,1))
);

ALTER TABLE PARTITS
ADD CONSTRAINT PARTITS_TEMPORADA_FK 
FOREIGN KEY(COMPETICIO,AAAA) 
REFERENCES TEMPORADES(COMPETICIO,AAAA);

ALTER TABLE PARTITS
ADD CONSTRAINT PARTIT_EQUIP1_FK 
FOREIGN KEY(EQUIP1) 
REFERENCES EQUIPS(CODI);

ALTER TABLE PARTITS
ADD CONSTRAINT PARTIT_EQUIP2_FK 
FOREIGN KEY(EQUIP2) 
REFERENCES EQUIPS(CODI);

--
-- JUGADORS
--

CREATE TABLE JUGADORS
(
  ID NUMBER CONSTRAINT JUGADORS_ID_NN NOT NULL 
, NOM VARCHAR2(20) CONSTRAINT JUGADORS_NOM_NN NOT NULL 
, COGNOM1 VARCHAR2(50) CONSTRAINT JUGADORS_COGNOM1_NN NOT NULL 
, COGNOM2 VARCHAR2(50) 
, NAIXEMENT DATE CONSTRAINT JUGADORS_NAIXEMENT_NN NOT NULL 
, NACIONALITAT VARCHAR2(3) CONSTRAINT JUGADORS_NACIONALITAT_NN NOT NULL
, CONSTRAINT JUGADORS_PK PRIMARY KEY(ID)
);

ALTER TABLE JUGADORS 
ADD CONSTRAINT JUGADORS_NACIONALITAT_FK 
FOREIGN KEY(NACIONALITAT) 
REFERENCES ZONES(CODI);

--
-- PLANTILLES 
--
-- EQUIP, JUGADOR, INICI, FINAL

CREATE TABLE PLANTILLES 
(
  EQUIP VARCHAR2(3) CONSTRAINT PLANTILLES_EQUIP_NN NOT NULL 
, JUGADOR NUMBER CONSTRAINT PLANTILLES_JUGADOR_NN NOT NULL 
, INICI DATE CONSTRAINT PLANTILLES_INICI_NN NOT NULL 
, FINAL DATE
, CONSTRAINT PLANTILLES_PK PRIMARY KEY(EQUIP, JUGADOR, INICI)
);

ALTER TABLE PLANTILLES 
ADD CONSTRAINT PLANTILLES_EQUIP_FK 
FOREIGN KEY(EQUIP) 
REFERENCES EQUIPS(CODI);

ALTER TABLE PLANTILLES 
ADD CONSTRAINT PLANTILLES_JUGADOR_FK 
FOREIGN KEY(JUGADOR) 
REFERENCES JUGADORS(ID);

--
-- ALINIACIONS
--
-- PARTIT, EQUIP, JUGADOR

--CREATE TABLE ALINIACIONS 
--(
--  PARTIT NUMBER CONSTRAINT ALINIACIONS_PARTIT_NN NOT NULL 
--, EQUIP VARCHAR2(3) CONSTRAINT ALINIACIONS_EQUIP_NN NOT NULL 
--, JUGADOR NUMBER CONSTRAINT ALINIACIONS_JUGADOR_NN NOT NULL 
--, CONSTRAINT ALINIACIONS_PK PRIMARY KEY(PARTIT, EQUIP, JUGADOR)
--);

--
-- TIPUS_GOL
--

CREATE TABLE TIPUS_GOL
(
  CODI VARCHAR(1) CONSTRAINT TIPUS_GOL_CODI_NN NOT NULL
, DESCRIPCIO VARCHAR2(20) CONSTRAINT TIPUS_GOL_DECRIPCIO_NN NOT NULL
, CONSTRAINT TIPUS_GOL_PK PRIMARY KEY(CODI)
);

--
-- GOL
--
-- SUPOSO QUE EL TEMPS AFEGIT NO SOBREPASSA ELS 5 MINUTS EN CAP DE LES DUES PARTS

CREATE TABLE GOLS 
(
  ID NUMBER CONSTRAINT GOLS_ID_NN NOT NULL 
, PARTIT NUMBER CONSTRAINT GOLS_PARTIT_NN NOT NULL
, EQUIP VARCHAR2(3) CONSTRAINT GOLS_EQUIP_NN NOT NULL 
, JUGADOR NUMBER CONSTRAINT GOLS_JUGADOR_NN NOT NULL 
, PART NUMBER CONSTRAINT GOLS_PART_NN NOT NULL 
, MINUT NUMBER CONSTRAINT GOLS_MINUT_NN NOT NULL 
, TIPUS VARCHAR2(1) 
, CONSTRAINT GOLS_PK PRIMARY KEY(ID)
, CONSTRAINT GOLS_PART_CK CHECK (PART IN (1,2))
, CONSTRAINT GOLS_MINUT_CK CHECK (MINUT BETWEEN 1 AND 95)
, CONSTRAINT GOLS_AFEGITS_CK CHECK ((PART=1 AND MINUT BETWEEN 1 AND 50) OR (PART=2 AND MINUT BETWEEN 45 AND 95))
);

ALTER TABLE GOLS 
ADD CONSTRAINT GOLS_PARTIT_FK 
FOREIGN KEY(PARTIT) 
REFERENCES PARTITS(ID);

ALTER TABLE GOLS 
ADD CONSTRAINT GOLS_EQUIP_FK 
FOREIGN KEY(EQUIP) 
REFERENCES EQUIPS(CODI);

ALTER TABLE GOLS 
ADD CONSTRAINT GOLS_JUGADOR_FK 
FOREIGN KEY(JUGADOR) 
REFERENCES JUGADORS(ID);

ALTER TABLE GOLS 
ADD CONSTRAINT GOLS_TIPUS_FK 
FOREIGN KEY(TIPUS) 
REFERENCES TIPUS_GOL(CODI);

--
-- USUARIS
--

CREATE TABLE USUARIS
(
  NOM_USUARI VARCHAR2(50) CONSTRAINT USUARIS_NOM_USUARI_NN NOT NULL
, CONTRASENYA VARCHAR2(20) CONSTRAINT USUARIS_CONTRASENYA_NN NOT NULL
, ACTIU VARCHAR(1) DEFAULT '1' CONSTRAINT USUARIS_ACTIU_NN NOT NULL
, IBAN VARCHAR(24) 
, TARGETA VARCHAR(20) 
, SALDO FLOAT DEFAULT 0.00 CONSTRAINT USUARIS_SALDO_NN NOT NULL
, CONSTRAINT USUARIS_PK PRIMARY KEY(NOM_USUARI)
, CONSTRAINT USUARIS_CCC_CK CHECK ((IBAN IS NOT NULL) OR (TARGETA IS NOT NULL))
);

--
-- USUARIS LOPD
--

CREATE TABLE USUARIS_LOPD
(
  NOM_USUARI VARCHAR2(50) CONSTRAINT USUARIS_LOPD_NOM_USUARI_NN NOT NULL
, NOM VARCHAR2(20) CONSTRAINT USUARIS_LOPD_NOM_NN NOT NULL 
, COGNOM1 VARCHAR2(50) CONSTRAINT USUARIS_LOPD_COGNOM1_NN NOT NULL 
, COGNOM2 VARCHAR2(50) 
, SEXE VARCHAR2(1) CONSTRAINT USUARIS_LOPD_SEXE_NN NOT NULL 
, IDENTIFICADOR VARCHAR2(1) CONSTRAINT USUARIS_LOPD_IDENTIFICADOR_NN NOT NULL 
, DNI VARCHAR2(9) CONSTRAINT USUARIS_LOPD_DNI_NN NOT NULL  
, NAIXEMENT DATE CONSTRAINT USUARIS_LOPD_NAIXEMENT_NN NOT NULL 
, NACIONALITAT VARCHAR2(3) CONSTRAINT USUARIS_LOPD_NACIONALITAT_NN NOT NULL
, RESIDENCIA VARCHAR2(3) CONSTRAINT USUARIS_LOPD_RESIDENCIA_NN NOT NULL
, CONSTRAINT USUARIS_LOPD_PK PRIMARY KEY(NOM_USUARI)
, CONSTRAINT USUARIS_LOPD_DNI_UK UNIQUE(DNI)
, CONSTRAINT USUARIS_LOPD_SEXE_CK CHECK (SEXE IN ('H', 'D'))
, CONSTRAINT USUARIS_LOPD_IDENTIFICADOR_CK CHECK (IDENTIFICADOR IN ('D', 'N', 'P'))
--, CONSTRAINT USUARIS_LOPD_NAIXEMENT_CK CHECK (YEAR(NOW()-NAIXEMENT) >= 18)
);

ALTER TABLE USUARIS_LOPD
ADD CONSTRAINT USUARIS_LOPD_USUARIS_UK
FOREIGN KEY(NOM_USUARI)
REFERENCES USUARIS(NOM_USUARI);

ALTER TABLE USUARIS_LOPD 
ADD CONSTRAINT USUARIS_LOPD_NACIONALITAT_FK 
FOREIGN KEY(NACIONALITAT) 
REFERENCES ZONES(CODI);

ALTER TABLE USUARIS_LOPD 
ADD CONSTRAINT USUARIS_LOPD_RESIDENCIA_FK 
FOREIGN KEY(RESIDENCIA) 
REFERENCES ZONES(CODI);

--
-- MODALITATS
-- 

CREATE TABLE MODALITATS
(
  ID NUMBER CONSTRAINT MODALITATS_ID_NN NOT NULL 
, DESCRIPCIO VARCHAR2(100) CONSTRAINT MODALITATS_DESCRIPCIO_NN NOT NULL 
, CONSTRAINT MODALITATS_PK PRIMARY KEY(ID)
);

--
-- COEFICIENTS
--

CREATE TABLE COEFICIENTS
(
  MODALITAT NUMBER CONSTRAINT COEFICIENTS_MODALITAT_NN NOT NULL 
, PARTIT NUMBER CONSTRAINT COEFICIENTS_PARTIT_NN NOT NULL 
, RESULTAT VARCHAR2(100)
, VALOR FLOAT DEFAULT 0.00 CONSTRAINT COEFICIENTS_VALOR_NN NOT NULL 
, CONSTRAINT COEFICIENTS_PK PRIMARY KEY(MODALITAT, PARTIT)
, CONSTRAINT COEFICIENTS_IMPORT_CK CHECK (VALOR > 0.00)
);

ALTER TABLE COEFICIENTS
ADD CONSTRAINT COEFICIENTS_MODALITAT_FK 
FOREIGN KEY(MODALITAT) 
REFERENCES MODALITATS(ID);

ALTER TABLE COEFICIENTS
ADD CONSTRAINT COEFICIENTS_PARTIT_FK 
FOREIGN KEY(PARTIT) 
REFERENCES PARTITS(ID);

-- ********************
-- RECTIFICACIO
-- ********************
DROP TABLE COEFICIENTS;

CREATE TABLE COEFICIENTS
(
  MODALITAT NUMBER CONSTRAINT COEFICIENTS_MODALITAT_NN NOT NULL 
, PARTIT NUMBER CONSTRAINT COEFICIENTS_PARTIT_NN NOT NULL 
, RESULTAT VARCHAR2(100) CONSTRAINT COEFICIENTS_RESULTAT_NN NOT NULL 
, VALOR FLOAT DEFAULT 0.00 CONSTRAINT COEFICIENTS_VALOR_NN NOT NULL 
, CONSTRAINT COEFICIENTS_PK PRIMARY KEY(MODALITAT, PARTIT, RESULTAT)
, CONSTRAINT COEFICIENTS_IMPORT_CK CHECK (VALOR > 0.00)
);

ALTER TABLE COEFICIENTS
ADD CONSTRAINT COEFICIENTS_MODALITAT_FK 
FOREIGN KEY(MODALITAT) 
REFERENCES MODALITATS(ID);

ALTER TABLE COEFICIENTS
ADD CONSTRAINT COEFICIENTS_PARTIT_FK 
FOREIGN KEY(PARTIT) 
REFERENCES PARTITS(ID);


--
-- APOSTES
--

CREATE TABLE APOSTES
(
  ID NUMBER CONSTRAINT APOSTES_ID_NN NOT NULL 
, NOM_USUARI VARCHAR2(50) CONSTRAINT APOSTES_USUARI_NN NOT NULL 
, MODALITAT NUMBER CONSTRAINT APOSTES_MODALITAT_NN NOT NULL 
, PARTIT NUMBER CONSTRAINT APOSTES_PARTIT_NN NOT NULL 
, APOSTA VARCHAR2(100) CONSTRAINT APOSTES_APOSTA_NN NOT NULL 
, IMPORT FLOAT DEFAULT 0.00 CONSTRAINT APOSTES_IMPORT_NN NOT NULL 
, RESOLTA VARCHAR(1) DEFAULT '0' CONSTRAINT APOSTES_RESOLTA_NN NOT NULL 
, GUANYADORA VARCHAR(1) DEFAULT '0' CONSTRAINT APOSTES_GUANYADORA_NN NOT NULL 
, CONSTRAINT APOSTES_PK PRIMARY KEY(ID)
, CONSTRAINT APOSTES_IMPORT_CK CHECK (IMPORT > 0.00)
, CONSTRAINT APOSTES_RESOLTA_CK CHECK (RESOLTA IN (0,1))
, CONSTRAINT APOSTES_GUANYADORA_CK CHECK (GUANYADORA IN (0,1))

);

ALTER TABLE APOSTES
ADD CONSTRAINT APOSTES_USUARI_FK 
FOREIGN KEY(NOM_USUARI) 
REFERENCES USUARIS(NOM_USUARI);

ALTER TABLE APOSTES
ADD CONSTRAINT APOSTES_MODALITAT_FK 
FOREIGN KEY(MODALITAT) 
REFERENCES MODALITATS(ID);

ALTER TABLE APOSTES
ADD CONSTRAINT APOSTES_PARTIT_FK 
FOREIGN KEY(PARTIT) 
REFERENCES PARTITS(ID);

--
-- RESULTATS
--

CREATE TABLE RESULTATS
(
  MODALITAT NUMBER CONSTRAINT RESULTATS_MODALITAT_NN NOT NULL 
, PARTIT NUMBER CONSTRAINT RESULTATS_PARTIT_NN NOT NULL 
, RESULTAT VARCHAR2(100)
, CONSTRAINT RESULTATS_PK PRIMARY KEY(MODALITAT, PARTIT)
);

ALTER TABLE RESULTATS
ADD CONSTRAINT RESULTATS_MODALITAT_FK 
FOREIGN KEY(MODALITAT) 
REFERENCES MODALITATS(ID);

ALTER TABLE RESULTATS
ADD CONSTRAINT RESULTATS_PARTIT_FK 
FOREIGN KEY(PARTIT) 
REFERENCES PARTITS(ID);


--
-- LOG_ACCIONS
--

CREATE TABLE LOG_ACCIONS
(
  USUARI VARCHAR2(50)
, ACCIO VARCHAR2(20)
, INFO VARCHAR2(4000)
, DATA DATE
, RESULTAT NUMBER DEFAULT '0'
);

