--
-- PKG_USUARIS
-- 
CREATE OR REPLACE PACKAGE PKG_USUARIS AS

  NOM_USUARI VARCHAR2(50);
--
-- ALTA_USUARI
-- AFEGEIX UN USUARI AL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION ALTA_USUARI 
          (V_NOM_USUARI VARCHAR2, V_CONTRASENYA VARCHAR2, V_NOM VARCHAR2, V_COGNOM1 VARCHAR2, V_COGNOM2 VARCHAR2, 
           V_SEXE VARCHAR2, V_IDENTIFICADOR VARCHAR2, V_DNI VARCHAR2, V_NAIXEMENT DATE, V_NACIONALITAT VARCHAR2, V_RESIDENCIA VARCHAR2,
           V_IBAN VARCHAR, V_TARGETA VARCHAR
          ) RETURN NUMBER;
--
-- BAIXA_USUARI
-- ELIMINA UN USUARI DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION BAIXA_USUARI (V_NOM_USUARI VARCHAR2) RETURN NUMBER;
--
-- MODIFICA_USUARI
-- MODIFICA UN USUARI DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION MODIFICA_USUARI
          (V_NOM_USUARI VARCHAR2, V_CONTRASENYA VARCHAR2, V_NOM VARCHAR2, V_COGNOM1 VARCHAR2, V_COGNOM2 VARCHAR2, 
           V_SEXE VARCHAR2, V_IDENTIFICADOR VARCHAR2, V_DNI VARCHAR2, V_NAIXEMENT DATE, V_NACIONALITAT VARCHAR2, V_RESIDENCIA VARCHAR2,
           V_ACTIU VARCHAR, V_IBAN VARCHAR, V_TARGETA VARCHAR
          ) RETURN NUMBER;
--
-- LLISTA_USUARIS
-- RETORNA LA LLISTA DE USUARIS
-- S'HA D'EXECUTAR AIX:
--   sql> variable c refcursor
--   sql> exec :c := pkg_usuaris.llista_usuaris();
--   sql> print c;
--
-- REFERENCIA:
--   http://dba.stackexchange.com/questions/18256/return-many-rows-on-a-plsql-oracle10g
--
  FUNCTION LLISTA_USUARIS RETURN SYS_REFCURSOR;
--
-- USUARI
-- RETORNA UN USUARI SEGONS LA SEVA CLAU PRIMARIA
--
  FUNCTION USUARI (V_NOM_USUARI VARCHAR2) RETURN USUARIS%ROWTYPE;
--
-- USUARI_LOPD
-- RETORNA UN USUARI_LOPD SEGONS LA SEVA CLAU PRIMARIA
--
  FUNCTION USUARI_LOPD (V_NOM_USUARI VARCHAR2) RETURN USUARIS_LOPD%ROWTYPE;
  
END PKG_USUARIS;
/

--
-- PKG_USUARIS BODY
-- 
CREATE OR REPLACE PACKAGE BODY PKG_USUARIS AS
  FUNCTION ALTA_USUARI 
          (V_NOM_USUARI VARCHAR2, V_CONTRASENYA VARCHAR2, V_NOM VARCHAR2, V_COGNOM1 VARCHAR2, V_COGNOM2 VARCHAR2, 
           V_SEXE VARCHAR2, V_IDENTIFICADOR VARCHAR2, V_DNI VARCHAR2, V_NAIXEMENT DATE, V_NACIONALITAT VARCHAR2, V_RESIDENCIA VARCHAR2,
           V_IBAN VARCHAR, V_TARGETA VARCHAR
          ) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        INSERT INTO USUARIS
        VALUES (V_NOM_USUARI, V_CONTRASENYA , '1', V_IBAN, V_TARGETA, 0.0);
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'USUARI ' || V_NOM_USUARI, '0');
        BEGIN
          INSERT INTO USUARIS_LOPD
          VALUES (V_NOM_USUARI, V_NOM, V_COGNOM1, V_COGNOM2, V_SEXE, V_IDENTIFICADOR, V_DNI, V_NAIXEMENT, V_NACIONALITAT, V_RESIDENCIA);
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'USUARI_LOPD ' || V_NOM_USUARI, '0');
        EXCEPTION 
          WHEN OTHERS THEN
            RETORN := SQLCODE;
            PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'USUARI_LOPD ' || V_NOM_USUARI, 'SQLCODE');
        END;
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'USUARI ' || V_NOM_USUARI, 'SQLCODE');
      END;
      RETURN RETORN;
    END;
  END ALTA_USUARI;
          
  FUNCTION BAIXA_USUARI (V_NOM_USUARI VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        DELETE FROM USUARIS_LOPD U_L
        WHERE U_L.NOM_USUARI = V_NOM_USUARI;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'USUARI_LOPD ' || V_NOM_USUARI, '0');
        BEGIN
          DELETE FROM USUARIS U
          WHERE U.NOM_USUARI = V_NOM_USUARI;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'USUARI ' || V_NOM_USUARI, '0');
          EXCEPTION 
          WHEN OTHERS THEN
            RETORN := SQLCODE;
            PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'USUARI ' || V_NOM_USUARI, 'SQLCODE');
          END;
        EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'USUARI_LOPD ' || V_NOM_USUARI, 'SQLCODE');
      END;
      RETURN RETORN;
    END;
  END BAIXA_USUARI;
  
  FUNCTION MODIFICA_USUARI
          (V_NOM_USUARI VARCHAR2, V_CONTRASENYA VARCHAR2, V_NOM VARCHAR2, V_COGNOM1 VARCHAR2, V_COGNOM2 VARCHAR2, 
           V_SEXE VARCHAR2, V_IDENTIFICADOR VARCHAR2, V_DNI VARCHAR2, V_NAIXEMENT DATE, V_NACIONALITAT VARCHAR2, V_RESIDENCIA VARCHAR2,
           V_ACTIU VARCHAR, V_IBAN VARCHAR, V_TARGETA VARCHAR
          ) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        UPDATE USUARIS U 
        SET U.CONTRASENYA = V_CONTRASENYA,
            U.ACTIU = V_ACTIU,
            U.IBAN = V_IBAN,
            U.TARGETA = V_TARGETA
        WHERE U.NOM_USUARI = V_NOM_USUARI;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'USUARI ' || V_NOM_USUARI, '0');
        BEGIN
          UPDATE USUARIS_LOPD U_L
          SET U_L.NOM = V_NOM, 
              U_L.COGNOM1 = V_COGNOM1, 
              U_L.COGNOM2 = V_COGNOM2, 
              U_L.SEXE = V_SEXE, 
              U_L.IDENTIFICADOR = V_IDENTIFICADOR, 
              U_L.DNI = V_DNI, 
              U_L.NAIXEMENT = V_NAIXEMENT, 
              U_L.NACIONALITAT = V_NACIONALITAT, 
              U_L.RESIDENCIA = V_RESIDENCIA
          WHERE U_L.NOM_USUARI = V_NOM_USUARI;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'USUARI _LOPD' || V_NOM_USUARI, '0');
        EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
            PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'USUARI_LOPD ' || V_NOM_USUARI, 'SQLCODE');
        END;
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'USUARI ' || V_NOM_USUARI, 'SQLCODE');
      END;
      RETURN RETORN;
    END;
  END MODIFICA_USUARI;
  
  FUNCTION LLISTA_USUARIS RETURN SYS_REFCURSOR IS
  BEGIN
    DECLARE
      RETORN SYS_REFCURSOR;
    BEGIN
      BEGIN
        OPEN RETORN FOR
        SELECT *
        FROM USUARIS U, USUARIS_LOPD U_L
        WHERE U.NOM_USUARI = U_L.NOM_USUARI;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'USUARIS', '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'USUARIS ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END LLISTA_USUARIS;

  FUNCTION USUARI (V_NOM_USUARI VARCHAR2) RETURN USUARIS%ROWTYPE IS
  BEGIN
    DECLARE
      RETORN USUARIS%ROWTYPE;
    BEGIN
      BEGIN
        SELECT *
        INTO RETORN
        FROM USUARIS U
        WHERE U.NOM_USUARI = V_NOM_USUARI;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'USUARIS ' || V_NOM_USUARI, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'USUARIS  ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END USUARI;

  FUNCTION USUARI_LOPD (V_NOM_USUARI VARCHAR2) RETURN USUARIS_LOPD%ROWTYPE IS
  BEGIN
    DECLARE
      RETORN USUARIS_LOPD%ROWTYPE;
    BEGIN
      BEGIN
        SELECT *
        INTO RETORN
        FROM USUARIS_LOPD U_L
        WHERE U_L.NOM_USUARI = V_NOM_USUARI;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'USUARIS_LOPD ' || V_NOM_USUARI, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'USUARIS_LOPD  ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END USUARI_LOPD;

END PKG_USUARIS;
/
