--
-- PKG_PARTITS
-- 
CREATE OR REPLACE PACKAGE PKG_PARTITS AS

--
-- ALTA_PARTIT
-- AFEGEIX UNA PARTIT AL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION ALTA_PARTIT (V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP1 VARCHAR2, V_EQUIP2 VARCHAR2, V_DATA DATE, V_JUGAT VARCHAR2) RETURN NUMBER;
  FUNCTION ALTA_PARTIT (V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP1 VARCHAR2, V_EQUIP2 VARCHAR2, V_DATA DATE, V_JUGAT VARCHAR2, V_ID IN OUT NUMBER) RETURN NUMBER;
--
-- BAIXA_PARTIT
-- ELIMINA UNA PARTIT DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION BAIXA_PARTIT (V_ID NUMBER) RETURN NUMBER;
--
-- MODIFICA_PARTIT
-- MODIFICA UNA PARTIT DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION MODIFICA_PARTIT (V_ID NUMBER, V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP1 VARCHAR2, V_EQUIP2 VARCHAR2, V_DATA DATE, V_JUGAT VARCHAR2) RETURN NUMBER;
--
-- LISTA_PARTITS
-- RETORNA LA LLISTA DE PARTITS
-- S'HA D'EXECUTAR AIX:
--   sql> variable c refcursor
--   sql> exec :c := pkg_partits.llista_partits();
--   sql> print c;
--
-- REFERENCIA:
--   http://dba.stackexchange.com/questions/18256/return-many-rows-on-a-plsql-oracle10g
--
  FUNCTION LLISTA_PARTITS RETURN SYS_REFCURSOR;
--
-- PARTIT
-- RETORNA UN PARTIT SEGONS LA SEVA CLAU PRIMARIA
--
  FUNCTION PARTIT (V_ID NUMBER) RETURN PARTITS%ROWTYPE;
--
-- CERCA_PARTIT
-- RETORNA UN PARTIT SEGONS LES SEVES DADES
--
  FUNCTION CERCA_PARTIT (V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP1 VARCHAR2, V_EQUIP2 VARCHAR2) RETURN NUMBER;



--
-- ALTA_TIPUS_GOL
-- AFEGEIX UN TIPUS_GOL AL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION ALTA_TIPUS_GOL (V_CODI VARCHAR2, V_DESCRIPCIO VARCHAR2) RETURN NUMBER;
--
-- BAIXA_TIPUS_GOL
-- ELIMINA UN TIPUS_GOL DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION BAIXA_TIPUS_GOL (V_CODI VARCHAR2) RETURN NUMBER;
--
-- MODIFICA_TIPUS_GOL
-- MODIFICA UN TIPUS_GOL DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION MODIFICA_TIPUS_GOL (V_CODI VARCHAR2, V_DESCRIPCIO VARCHAR2) RETURN NUMBER;
--
-- LLISTA_TIPUS_GOLS
-- RETORNA LA LLISTA DE TIPUS_GOLS
-- S'HA D'EXECUTAR AIX:
--   sql> variable c refcursor
--   sql> exec :c := pkg_partits.llista_tipus_gols();
--   sql> print c;
--
-- REFERENCIA:
--   http://dba.stackexchange.com/questions/18256/return-many-rows-on-a-plsql-oracle10g
--
  FUNCTION LLISTA_TIPUS_GOLS RETURN SYS_REFCURSOR;
--
-- TIP_GOL
-- RETORNA UN TIPUS_GOL SEGONS LA SEVA CLAU PRIMARIA
--
  FUNCTION TIP_GOL (V_CODI VARCHAR2) RETURN TIPUS_GOL%ROWTYPE;



--
-- ALTA_GOL
-- AFEGEIX UN GOL AL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION ALTA_GOL (V_PARTIT NUMBER, V_EQUIP VARCHAR2, V_JUGADOR NUMBER, V_PART NUMBER, V_MINUT NUMBER, V_TIPUS VARCHAR2) RETURN NUMBER;
  FUNCTION ALTA_GOL (V_PARTIT NUMBER, V_EQUIP VARCHAR2, V_JUGADOR NUMBER, V_PART NUMBER, V_MINUT NUMBER, V_TIPUS VARCHAR2, V_ID IN OUT NUMBER) RETURN NUMBER;
--
-- BAIXA_GOL
-- ELIMINA UN GOL DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION BAIXA_GOL (V_ID NUMBER) RETURN NUMBER;
--
-- MODIFICA_GOL
-- MODIFICA UN GOL DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION MODIFICA_GOL (V_ID NUMBER, V_PARTIT NUMBER, V_EQUIP VARCHAR2, V_JUGADOR NUMBER, V_PART NUMBER, V_MINUT NUMBER, V_TIPUS VARCHAR2) RETURN NUMBER;
--
-- LLISTA_GOLS
-- RETORNA LA LLISTA DE GOLS
-- S'HA D'EXECUTAR AIX:
--   sql> variable c refcursor
--   sql> exec :c := pkg_partits.llista_gols();
--   sql> print c;
--
-- REFERENCIA:
--   http://dba.stackexchange.com/questions/18256/return-many-rows-on-a-plsql-oracle10g
--
  FUNCTION LLISTA_GOLS RETURN SYS_REFCURSOR;
--
-- CERCA_GOL
-- RETORNA UN GOL SEGONS LES SEVES DADES
--
  FUNCTION CERCA_GOL (V_PARTIT NUMBER, V_PART NUMBER, V_MINUT NUMBER) RETURN NUMBER;
--
-- GOL
-- RETORNA UN GOL SEGONS LA SEVA CLAU PRIMARIA
--
  FUNCTION GOL (V_ID NUMBER) RETURN GOLS%ROWTYPE;

END PKG_PARTITS;
/

--
-- PKG_PARTITS BODY
-- 
CREATE OR REPLACE PACKAGE BODY PKG_PARTITS AS

--
-- PARTITS
--
  FUNCTION ALTA_PARTIT (V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP1 VARCHAR2, V_EQUIP2 VARCHAR2, V_DATA DATE, V_JUGAT VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
      V_ID NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        RETORN := ALTA_PARTIT (V_COMPETICIO, V_AAAA, V_EQUIP1, V_EQUIP2, V_DATA, V_JUGAT, V_ID);
      END;
      RETURN RETORN;
    END;
  END ALTA_PARTIT;

  FUNCTION ALTA_PARTIT (V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP1 VARCHAR2, V_EQUIP2 VARCHAR2, V_DATA DATE, V_JUGAT VARCHAR2, V_ID IN OUT NUMBER) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        INSERT INTO PARTITS
        VALUES (0, V_COMPETICIO, V_AAAA, V_EQUIP1, V_EQUIP2, V_DATA, '0')
        RETURNING ID INTO V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'PARTIT ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'PARTIT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END ALTA_PARTIT;
          
  FUNCTION BAIXA_PARTIT (V_ID NUMBER) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        DELETE FROM PARTITS P
        WHERE P.ID = V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'PARTIT ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'PARTIT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END BAIXA_PARTIT;
  
  FUNCTION MODIFICA_PARTIT (V_ID NUMBER, V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP1 VARCHAR2, V_EQUIP2 VARCHAR2, V_DATA DATE, V_JUGAT VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        UPDATE PARTITS P 
        SET P.COMPETICIO = V_COMPETICIO,
            P.AAAA = V_AAAA,
            P.EQUIP1 = V_EQUIP1,
            P.EQUIP2 = V_EQUIP2,
            P.DATA = V_DATA,
            P.JUGAT = V_JUGAT
        WHERE P.ID = V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'PARTIT ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'PARTIT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END MODIFICA_PARTIT;
  
  FUNCTION LLISTA_PARTITS RETURN SYS_REFCURSOR IS
  BEGIN
    DECLARE
      RETORN SYS_REFCURSOR;
    BEGIN
      BEGIN
        OPEN RETORN FOR
        SELECT *
        FROM PARTITS;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'PARTIT', '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'PARTIT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END LLISTA_PARTITS;

  FUNCTION PARTIT (V_ID NUMBER) RETURN PARTITS%ROWTYPE IS
  BEGIN
    DECLARE
      RETORN PARTITS%ROWTYPE;
    BEGIN
      BEGIN
        SELECT *
        INTO RETORN
        FROM PARTITS P
        WHERE P.ID = V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'PARTIT ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'PARTIT ID:' || V_ID || ' ERR:' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END PARTIT;
  
  FUNCTION CERCA_PARTIT (V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP1 VARCHAR2, V_EQUIP2 VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        SELECT MIN(P.ID)
        INTO RETORN
        FROM PARTITS P 
        WHERE P.COMPETICIO = V_COMPETICIO
          AND P.AAAA = V_AAAA
          AND P.EQUIP1 = V_EQUIP1
          AND P.EQUIP2 = V_EQUIP2;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'CERCA', 'PARTIT ' || RETORN || V_COMPETICIO || V_AAAA || V_EQUIP1 || V_EQUIP2, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'CERCA', 'PARTIT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END CERCA_PARTIT;



--
-- TIPUS_GOLS
--
  FUNCTION ALTA_TIPUS_GOL (V_CODI VARCHAR2, V_DESCRIPCIO VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        INSERT INTO TIPUS_GOL
        VALUES (V_CODI, V_DESCRIPCIO);
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'TIPUS_GOL ' || V_CODI, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'TIPUS_GOL ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END ALTA_TIPUS_GOL;
          
  FUNCTION BAIXA_TIPUS_GOL (V_CODI VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        DELETE FROM TIPUS_GOL TG
        WHERE TG.CODI = V_CODI;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'TIPUS_GOL ' || V_CODI, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'TIPUS_GOL ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END BAIXA_TIPUS_GOL;
  
  FUNCTION MODIFICA_TIPUS_GOL (V_CODI VARCHAR2, V_DESCRIPCIO VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        UPDATE TIPUS_GOL TG 
        SET TG.DESCRIPCIO = V_DESCRIPCIO
        WHERE TG.CODI = V_CODI;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'TIPUS_GOL ' || V_CODI, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'TIPUS_GOL ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END MODIFICA_TIPUS_GOL;
  
  FUNCTION LLISTA_TIPUS_GOLS RETURN SYS_REFCURSOR IS
  BEGIN
    DECLARE
      RETORN SYS_REFCURSOR;
    BEGIN
      BEGIN
        OPEN RETORN FOR
        SELECT *
        FROM TIPUS_GOL;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'TIPUS_GOL', '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'TIPUS_GOL ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END LLISTA_TIPUS_GOLS;

  FUNCTION TIP_GOL (V_CODI VARCHAR2) RETURN TIPUS_GOL%ROWTYPE IS
  BEGIN
    DECLARE
      RETORN TIPUS_GOL%ROWTYPE;
    BEGIN
      BEGIN
        SELECT *
        INTO RETORN
        FROM TIPUS_GOL T_G
        WHERE T_G.CODI = V_CODI;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'TIPUS_GOL ' || V_CODI, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'TIPUS_GOL ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END TIP_GOL;
  



--
-- GOLS
--
  FUNCTION ALTA_GOL (V_PARTIT NUMBER, V_EQUIP VARCHAR2, V_JUGADOR NUMBER, V_PART NUMBER, V_MINUT NUMBER, V_TIPUS VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
      V_ID NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        RETORN := ALTA_GOL (V_PARTIT, V_EQUIP, V_JUGADOR, V_PART, V_MINUT, V_TIPUS, V_ID);
      END;
      RETURN RETORN;
    END;
  END ALTA_GOL;

  FUNCTION ALTA_GOL (V_PARTIT NUMBER, V_EQUIP VARCHAR2, V_JUGADOR NUMBER, V_PART NUMBER, V_MINUT NUMBER, V_TIPUS VARCHAR2, V_ID IN OUT NUMBER) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        INSERT INTO GOLS
        VALUES (0, V_PARTIT, V_EQUIP, V_JUGADOR, V_PART, V_MINUT, V_TIPUS)
        RETURNING ID INTO V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'GOL ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'GOL ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END ALTA_GOL;
          
  FUNCTION BAIXA_GOL (V_ID NUMBER) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        DELETE FROM GOLS G
        WHERE G.ID = V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'GOL ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'GOL ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END BAIXA_GOL;
  
  FUNCTION MODIFICA_GOL (V_ID NUMBER, V_PARTIT NUMBER, V_EQUIP VARCHAR2, V_JUGADOR NUMBER, V_PART NUMBER, V_MINUT NUMBER, V_TIPUS VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        UPDATE GOLS G 
        SET G.PARTIT = V_PARTIT,
            G.EQUIP = V_EQUIP,
            G.JUGADOR = V_JUGADOR,
            G.PART = V_PART,
            G.MINUT = V_MINUT,
            G.TIPUS = V_TIPUS
        WHERE G.ID = V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'GOL ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'GOL ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END MODIFICA_GOL;
  
  FUNCTION LLISTA_GOLS RETURN SYS_REFCURSOR IS
  BEGIN
    DECLARE
      RETORN SYS_REFCURSOR;
    BEGIN
      BEGIN
        OPEN RETORN FOR
        SELECT *
        FROM GOLS;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'GOL', '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'GOL ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END LLISTA_GOLS;

  FUNCTION CERCA_GOL (V_PARTIT NUMBER, V_PART NUMBER, V_MINUT NUMBER) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        SELECT MIN(G.ID)
        INTO RETORN
        FROM GOLS G 
        WHERE G.PARTIT = V_PARTIT
          AND G.PART = V_PART
          AND G.MINUT = V_MINUT;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'CERCA', 'GOL ' || RETORN || V_PARTIT || V_PART || V_MINUT, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'CERCA', 'GOL ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END CERCA_GOL;

  FUNCTION GOL (V_ID NUMBER) RETURN GOLS%ROWTYPE IS
  BEGIN
    DECLARE
      RETORN GOLS%ROWTYPE;
    BEGIN
      BEGIN
        SELECT *
        INTO RETORN
        FROM GOLS G
        WHERE G.ID = V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'GOL' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'GOL ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END GOL;
  
END PKG_PARTITS;
/
