--
-- PKG_JOC
-- 
CREATE OR REPLACE PACKAGE PKG_JOC AS

--
-- LOGIN
-- VALIDACIO D'UN USUARI AL SISTEMA
--
  FUNCTION LOGIN(V_NOM_USUARI VARCHAR2, V_CONTRASENYA VARCHAR2) RETURN NUMBER;
--
-- LOGOFF
-- DESCONNEXIO D'UN USUARI DEL SISTEMA
--
  FUNCTION LOGOUT RETURN NUMBER;
--
-- APOSTAR
-- REALITZA UNA APOSTA D'UN USUARI DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION APOSTAR(V_MODALITAT NUMBER, V_PARTIT NUMBER, V_APOSTA VARCHAR2, V_IMPORT FLOAT) RETURN NUMBER;
--
-- HISTORIC_RESULTATS
-- RETORNA LA LLISTA HISTORICA DE RESULTATS PER COMPETICIO, TEMPORADA I EQUIPS
-- S'HA D'EXECUTAR AIX:
--   sql> variable c refcursor
--   sql> exec :c := pkg_usuaris.historic_resultats();
--   sql> print c;
--
-- REFERENCIA:
--   http://dba.stackexchange.com/questions/18256/return-many-rows-on-a-plsql-oracle10g
--
  FUNCTION HISTORIC_RESULTATS(V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP1 VARCHAR2, V_EQUIP2 VARCHAR2) RETURN SYS_REFCURSOR;
--
-- HISTORIC_GOLS
-- RETORNA LA LLISTA HISTORICA DE GOLS PER JUGADOR EN COMPETICIO, TEMPORADA I EQUIPS
-- S'HA D'EXECUTAR AIX:
--   sql> variable c refcursor
--   sql> exec :c := pkg_usuaris.historic_gols();
--   sql> print c;
--
-- REFERENCIA:
--   http://dba.stackexchange.com/questions/18256/return-many-rows-on-a-plsql-oracle10g
--
  FUNCTION HISTORIC_GOLS(V_JUGADOR NUMBER, V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP VARCHAR2) RETURN SYS_REFCURSOR;
--
-- HISTORIC_APOSTES
-- RETORNA LA LLISTA D'APOSTES D'UN JUGADOR PER  
-- S'HA D'EXECUTAR AIX:
--   sql> variable c refcursor
--   sql> exec :c := pkg_usuaris.historic_apostes();
--   sql> print c;
--
-- REFERENCIA:
--   http://dba.stackexchange.com/questions/18256/return-many-rows-on-a-plsql-oracle10g
--
  FUNCTION HISTORIC_APOSTES(V_MODALITAT VARCHAR2, V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP VARCHAR2) RETURN SYS_REFCURSOR;
  
END PKG_JOC;
/

--
-- PKG_JOC BODY
-- 
CREATE OR REPLACE PACKAGE BODY PKG_JOC AS

  FUNCTION LOGIN (V_NOM_USUARI VARCHAR2, V_CONTRASENYA VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        SELECT 1
        INTO RETORN
        FROM USUARIS U
        WHERE U.NOM_USUARI = V_NOM_USUARI
          AND U.CONTRASENYA = V_CONTRASENYA;
        IF RETORN = 0 THEN
          PKG_USUARIS.NOM_USUARI := 'no_validat';
          RETORN := -1;
        ELSE
          PKG_USUARIS.NOM_USUARI := V_NOM_USUARI;
          RETORN := 0;
        END IF;
        PKG_UTIL.AFEGIR_LOG (USER, 'LOGIN', 'USUARI ' || V_NOM_USUARI, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (USER, 'LOGIN', 'USUARI ' || SQLERRM, 'SQLCODE');
      END;
      RETURN RETORN;
    END;
  END LOGIN;
  
  FUNCTION LOGOUT RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      PKG_USUARIS.NOM_USUARI := 'no_user';
      RETURN RETORN;
    END;    
  END LOGOUT;
  
  FUNCTION APOSTAR (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_APOSTA VARCHAR2, V_IMPORT FLOAT) RETURN NUMBER IS
  BEGIN
    RETURN PKG_APOSTES.ALTA_APOSTA(V_MODALITAT, V_PARTIT, V_APOSTA, V_IMPORT);
  END;
          
  FUNCTION HISTORIC_RESULTATS(V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP1 VARCHAR2, V_EQUIP2 VARCHAR2) RETURN SYS_REFCURSOR IS
  BEGIN
    DECLARE
      RETORN SYS_REFCURSOR;
      QUERY VARCHAR2(32000);
      QUERYW VARCHAR2(32000);
    BEGIN
      QUERY := '';
      QUERYW := '';
      BEGIN
        QUERY := QUERY || 'OPEN RETORN FOR ';
        QUERY := QUERY || 'SELECT * ';
        QUERY := QUERY || 'FROM PARTITS P ';
        
        IF (V_COMPETICIO IS NOT NULL) THEN
         QUERYW := QUERYW || 'WHERE P.COMPETICIO = ''' || V_COMPETICIO || '''';
        END IF;

        IF (V_AAAA IS NOT NULL) THEN
          IF (QUERYW IS NULL) THEN
            QUERYW := QUERYW || 'WHERE ';
          ELSE
            QUERYW := QUERYW || 'AND ';
          END IF;
          QUERYW := QUERYW || 'P.AAAA = ' || V_AAAA;
        END IF;
        
        IF (V_EQUIP1 IS NOT NULL) THEN
          IF (QUERYW IS NULL) THEN
            QUERYW := QUERYW || 'WHERE ';
          ELSE
            QUERYW := QUERYW || 'AND ';
          END IF;
          QUERYW := QUERYW || 'P.EQUIP1 = ''' || V_EQUIP1 || '''';
        END IF;

        IF (V_EQUIP2 IS NOT NULL) THEN
          IF (QUERYW IS NULL) THEN
            QUERYW := QUERYW || 'WHERE ';
          ELSE
            QUERYW := QUERYW || 'AND ';
          END IF;
          QUERYW := QUERYW || 'P.EQUIP2 = ''' || V_EQUIP2 || '''';
        END IF;

        QUERY := QUERY || QUERYW;
        QUERY := QUERY || 'ORDER BY DATA DESC;';
        EXECUTE IMMEDIATE QUERY;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'HISTORIC_RESULTATS', '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'HISTORIC_RESULTATS ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END HISTORIC_RESULTATS;

  FUNCTION HISTORIC_GOLS(V_JUGADOR NUMBER, V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP VARCHAR2) RETURN SYS_REFCURSOR IS
  BEGIN
    DECLARE
      RETORN SYS_REFCURSOR;
      QUERY VARCHAR2(32000);
      QUERYW VARCHAR2(32000);
    BEGIN
      QUERY := '';
      QUERYW := '';
      BEGIN
        QUERY := QUERY || 'OPEN RETORN FOR ';
        QUERY := QUERY || 'SELECT * ';
        QUERY := QUERY || 'FROM GOLS G ';
        
        IF (V_JUGADOR IS NOT NULL) THEN
         QUERYW := QUERYW || 'WHERE G.JUGADOR = ''' || V_JUGADOR || '''';
        END IF;

        IF (V_COMPETICIO IS NOT NULL) THEN
          IF (QUERYW IS NULL) THEN
            QUERYW := QUERYW || 'WHERE ';
          ELSE
            QUERYW := QUERYW || 'AND ';
          END IF;
          QUERYW := QUERYW || 'G.COMPETICIO = ''' || V_COMPETICIO || '''';
        END IF;
        
        IF (V_AAAA IS NOT NULL) THEN
          IF (QUERYW IS NULL) THEN
            QUERYW := QUERYW || 'WHERE ';
          ELSE
            QUERYW := QUERYW || 'AND ';
          END IF;
          QUERYW := QUERYW || 'G.AAAA = ' || V_AAAA;
        END IF;

        IF (V_EQUIP IS NOT NULL) THEN
          IF (QUERYW IS NULL) THEN
            QUERYW := QUERYW || 'WHERE ';
          ELSE
            QUERYW := QUERYW || 'AND ';
          END IF;
          QUERYW := QUERYW || 'G.EQUIP = ''' || V_EQUIP || '''';
        END IF;

        QUERY := QUERY || QUERYW;
        QUERY := QUERY || 'ORDER BY DATA DESC;';
        EXECUTE IMMEDIATE QUERY;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'HISTORIC_GOLS', '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'HISTORIC_GOLS ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END HISTORIC_GOLS;

  FUNCTION HISTORIC_APOSTES(V_MODALITAT VARCHAR2, V_COMPETICIO VARCHAR2, V_AAAA NUMBER, V_EQUIP VARCHAR2) RETURN SYS_REFCURSOR IS
  BEGIN
    DECLARE
      RETORN SYS_REFCURSOR;
      QUERY VARCHAR2(32000);
      QUERYW VARCHAR2(32000);
    BEGIN
      QUERY := '';
      QUERYW := '';
      BEGIN
        QUERY := QUERY || 'OPEN RETORN FOR ';
        QUERY := QUERY || 'SELECT * ';
        QUERY := QUERY || 'FROM APOSTES A ';
        
        IF (V_MODALITAT IS NOT NULL) THEN
         QUERYW := QUERYW || 'WHERE A.MODALITAT = ''' || V_MODALITAT || '''';
        END IF;

        IF (V_COMPETICIO IS NOT NULL) THEN
          IF (QUERYW IS NULL) THEN
            QUERYW := QUERYW || 'WHERE ';
          ELSE
            QUERYW := QUERYW || 'AND ';
          END IF;
          QUERYW := QUERYW || 'A.COMPETICIO = ''' || V_COMPETICIO || '''';
        END IF;
        
        IF (V_AAAA IS NOT NULL) THEN
          IF (QUERYW IS NULL) THEN
            QUERYW := QUERYW || 'WHERE ';
          ELSE
            QUERYW := QUERYW || 'AND ';
          END IF;
          QUERYW := QUERYW || 'A.AAAA = ' || V_AAAA;
        END IF;

        IF (V_EQUIP IS NOT NULL) THEN
          IF (QUERYW IS NULL) THEN
            QUERYW := QUERYW || 'WHERE ';
          ELSE
            QUERYW := QUERYW || 'AND ';
          END IF;
          QUERYW := QUERYW || 'A.EQUIP = ''' || V_EQUIP || '''';
        END IF;

        IF (PKG_USUARIS.NOM_USUARI IS NOT NULL) THEN
          IF (QUERYW IS NULL) THEN
            QUERYW := QUERYW || 'WHERE ';
          ELSE
            QUERYW := QUERYW || 'AND ';
          END IF;
          QUERYW := QUERYW || 'A.NOM_USUARI = ''' || PKG_USUARIS.NOM_USUARI || '''';
        END IF;

        QUERY := QUERY || QUERYW;
        QUERY := QUERY || 'ORDER BY DATA DESC;';
        EXECUTE IMMEDIATE QUERY;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'HISTORIC_APOSTES', '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'HISTORIC_APOSTES ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END HISTORIC_APOSTES;

END PKG_JOC;
/

