--
-- PKG_APOSTES
-- 
CREATE OR REPLACE PACKAGE PKG_APOSTES AS

--
-- ALTA_MODALITAT
-- AFEGEIX UN MODALITAT AL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION ALTA_MODALITAT (V_DESCRIPCIO VARCHAR2) RETURN NUMBER;
  FUNCTION ALTA_MODALITAT (V_DESCRIPCIO VARCHAR2, V_ID IN OUT NUMBER) RETURN NUMBER;
--
-- BAIXA_MODALITAT
-- ELIMINA UN MODALITAT DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION BAIXA_MODALITAT (V_ID NUMBER) RETURN NUMBER;
--
-- MODIFICA_MODALITAT
-- MODIFICA UN MODALITAT DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION MODIFICA_MODALITAT (V_ID NUMBER, V_DESCRIPCIO VARCHAR2) RETURN NUMBER;
--
-- LLISTA_MODALITATS
-- RETORNA LA LLISTA DE MODALITATS
-- S'HA D'EXECUTAR AIX:
--   sql> variable c refcursor
--   sql> exec :c := pkg_apostes.llista_modalitats();
--   sql> print c;
--
-- REFERENCIA:
--   http://dba.stackexchange.com/questions/18256/return-many-rows-on-a-plsql-oracle10g
--
  FUNCTION LLISTA_MODALITATS RETURN SYS_REFCURSOR;
--
-- MODALITAT
-- RETORNA UNA MODALITAT SEGONS LA SEVA CLAU PRIMARIA
--
  FUNCTION MODALITAT (V_ID NUMBER) RETURN MODALITATS%ROWTYPE;
--
-- CERCA_MODALITAT
-- RETORNA UNA MODALITAT SEGONS LES SEVES DADES
--
  FUNCTION CERCA_MODALITAT (V_DESCRIPCIO VARCHAR2) RETURN NUMBER;



--
-- ALTA_COEFICIENT
-- AFEGEIX UN COEFICIENT AL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION ALTA_COEFICIENT (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_RESULTAT VARCHAR2, V_VALOR FLOAT) RETURN NUMBER;
--
-- BAIXA_COEFICIENT
-- ELIMINA UN COEFICIENT DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION BAIXA_COEFICIENT (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_RESULTAT VARCHAR2) RETURN NUMBER;
--
-- MODIFICA_COEFICIENT
-- MODIFICA UN COEFICIENT DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION MODIFICA_COEFICIENT (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_RESULTAT VARCHAR2, V_VALOR FLOAT) RETURN NUMBER;
--
-- LLISTA_COEFICIENTS
-- RETORNA LA LLISTA DE COEFICIENTS
-- S'HA D'EXECUTAR AIX:
--   sql> variable c refcursor
--   sql> exec :c := pkg_apostes.llista_coeficients();
--   sql> print c;
--
-- REFERENCIA:
--   http://dba.stackexchange.com/questions/18256/return-many-rows-on-a-plsql-oracle10g
--
  FUNCTION LLISTA_COEFICIENTS RETURN SYS_REFCURSOR;
--
-- COEFICIENT
-- RETORNA UNA COEFICIENT SEGONS LA SEVA CLAU PRIMARIA
--
  FUNCTION COEFICIENT (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_RESULTAT VARCHAR2) RETURN COEFICIENTS%ROWTYPE;



--
-- ALTA_APOSTA
-- AFEGEIX UNA APOSTA AL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION ALTA_APOSTA (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_APOSTA VARCHAR2, V_IMPORT FLOAT, V_ID IN OUT NUMBER) RETURN NUMBER;
  FUNCTION ALTA_APOSTA (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_APOSTA VARCHAR2, V_IMPORT FLOAT) RETURN NUMBER;
--
-- BAIXA_APOSTA
-- ELIMINA UNA APOSTA DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION BAIXA_APOSTA (V_ID NUMBER) RETURN NUMBER;
--
-- MODIFICA_APOSTA
-- MODIFICA UNA APOSTA DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION MODIFICA_APOSTA (V_ID NUMBER, V_APOSTA VARCHAR2, V_IMPORT FLOAT) RETURN NUMBER;
--
-- LLISTA_APOSTES
-- RETORNA LA LLISTA D'APOSTES
-- S'HA D'EXECUTAR AIX:
--   sql> variable c refcursor
--   sql> exec :c := pkg_apostes.llista_apostes();
--   sql> print c;
--
-- REFERENCIA:
--   http://dba.stackexchange.com/questions/18256/return-many-rows-on-a-plsql-oracle10g
--
  FUNCTION LLISTA_APOSTES RETURN SYS_REFCURSOR;
--
-- APOSTA
-- RETORNA UN APOSTA SEGONS LA SEVA CLAU PRIMARIA
--
  FUNCTION APOSTA (V_ID NUMBER) RETURN APOSTES%ROWTYPE;
--
-- CERCA_APOSTA
-- RETORNA UNA APOSTA SEGONS LES SEVES DADES
--
  FUNCTION CERCA_APOSTA (V_MODALITAT NUMBER, V_PARTIT NUMBER) RETURN NUMBER;



--
-- ALTA_RESULTAT
-- AFEGEIX UN RESULTAT AL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION ALTA_RESULTAT (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_RESULTAT VARCHAR2) RETURN NUMBER;
--
-- BAIXA_RESULTAT
-- ELIMINA UN RESULTAT DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION BAIXA_RESULTAT (V_MODALITAT NUMBER, V_PARTIT NUMBER) RETURN NUMBER;
--
-- MODIFICA_RESULTAT
-- MODIFICA UN RESULTAT DEL SISTEMA O TORNA EL CODI D'ERROR QUE S'HA PRODUIT
--
  FUNCTION MODIFICA_RESULTAT (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_RESULTAT VARCHAR2) RETURN NUMBER;
--
-- LLISTA_RESULTATS
-- RETORNA LA LLISTA DE RESULTATS
-- S'HA D'EXECUTAR AIX:
--   sql> variable c refcursor
--   sql> exec :c := pkg_apostes.llista_resultats();
--   sql> print c;
--
-- REFERENCIA:
--   http://dba.stackexchange.com/questions/18256/return-many-rows-on-a-plsql-oracle10g
--
  FUNCTION LLISTA_RESULTATS RETURN SYS_REFCURSOR;
--
-- RESULTAT
-- RETORNA UN RESULTAT SEGONS LA SEVA CLAU PRIMARIA
--
  FUNCTION RESULTAT (V_MODALITAT NUMBER, V_PARTIT NUMBER) RETURN RESULTATS%ROWTYPE;



--
-- RESOLDRE_APOSTES
-- RESOL LES APOSTES DELS PARTITS QUE JA S'HAN JUGAT I ASIGNA ELS DINERS QUE CAL PAGAR I ELS ACUMULA A L'USUARI.
--
  PROCEDURE RESOLDRE_APOSTES;
  
END PKG_APOSTES;
/

--
-- PKG_APOSTES BODY
-- 
CREATE OR REPLACE PACKAGE BODY PKG_APOSTES AS

--
-- MODALITATS
--
  FUNCTION ALTA_MODALITAT (V_DESCRIPCIO VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
      V_ID NUMBER;
    BEGIN
      RETORN := 0;
      V_ID := 0;
      BEGIN
        RETORN := ALTA_MODALITAT(V_DESCRIPCIO, V_ID);
      END;
      RETURN RETORN;
    END;
  END ALTA_MODALITAT;
  
  FUNCTION ALTA_MODALITAT (V_DESCRIPCIO VARCHAR2, V_ID IN OUT NUMBER) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        INSERT INTO MODALITATS
        VALUES (0, V_DESCRIPCIO)
        RETURNING ID INTO V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'MODALITAT ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'MODALITAT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END ALTA_MODALITAT;
          
  FUNCTION BAIXA_MODALITAT (V_ID NUMBER) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        DELETE FROM MODALITATS M
        WHERE M.ID = V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'MODALITAT ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'MODALITAT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END BAIXA_MODALITAT;
  
  FUNCTION MODIFICA_MODALITAT (V_ID NUMBER, V_DESCRIPCIO VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        UPDATE MODALITATS M 
        SET M.DESCRIPCIO = V_DESCRIPCIO
        WHERE M.ID = V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'MODALITAT ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'MODALITAT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END MODIFICA_MODALITAT;
  
  FUNCTION LLISTA_MODALITATS RETURN SYS_REFCURSOR IS
  BEGIN
    DECLARE
      RETORN SYS_REFCURSOR;
    BEGIN
      BEGIN
        OPEN RETORN FOR
        SELECT *
        FROM MODALITATS;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'MODALITAT', '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'MODALITAT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END LLISTA_MODALITATS;

  FUNCTION MODALITAT (V_ID NUMBER) RETURN MODALITATS%ROWTYPE IS
  BEGIN
    DECLARE
      RETORN MODALITATS%ROWTYPE;
    BEGIN
      BEGIN
        SELECT *
        INTO RETORN
        FROM MODALITATS M
        WHERE M.ID = V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'MODALITAT ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'MODALITAT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END MODALITAT;
  
  FUNCTION CERCA_MODALITAT (V_DESCRIPCIO VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        SELECT MIN(M.ID)
        INTO RETORN
        FROM MODALITATS M 
        WHERE M.DESCRIPCIO = V_DESCRIPCIO;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'CERCA', 'MODALITAT ' || V_DESCRIPCIO, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'CERCA', 'MODALITAT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END CERCA_MODALITAT;



--
-- COEFICIENTS
--
  FUNCTION ALTA_COEFICIENT (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_RESULTAT VARCHAR2, V_VALOR FLOAT) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        INSERT INTO COEFICIENTS
        VALUES (V_MODALITAT, V_PARTIT, V_RESULTAT, V_VALOR);
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'COEFICIENTS ' || V_MODALITAT || ',' || V_PARTIT || ',' || V_RESULTAT, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'COEFICIENTS ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END ALTA_COEFICIENT;
          
  FUNCTION BAIXA_COEFICIENT (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_RESULTAT VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        DELETE FROM COEFICIENTS C
        WHERE C.MODALITAT = V_MODALITAT
          AND C.PARTIT = V_PARTIT
          AND C.RESULTAT = V_RESULTAT;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'COEFICIENTS ' || V_MODALITAT || ',' || V_PARTIT || ',' || V_RESULTAT, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'COEFICIENTS ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END BAIXA_COEFICIENT;
  
  FUNCTION MODIFICA_COEFICIENT (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_RESULTAT VARCHAR2, V_VALOR FLOAT) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        UPDATE COEFICIENTS C 
        SET C.VALOR = V_VALOR
        WHERE C.MODALITAT = V_MODALITAT
          AND C.PARTIT = V_PARTIT
          AND C.RESULTAT = V_RESULTAT;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'COEFICIENTS ' || V_MODALITAT || ',' || V_PARTIT || ',' || V_RESULTAT, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'COEFICIENTS ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END MODIFICA_COEFICIENT;
  
  FUNCTION LLISTA_COEFICIENTS RETURN SYS_REFCURSOR IS
  BEGIN
    DECLARE
      RETORN SYS_REFCURSOR;
    BEGIN
      BEGIN
        OPEN RETORN FOR
        SELECT *
        FROM COEFICIENTS;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'COEFICIENTS', '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'COEFICIENTS ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END LLISTA_COEFICIENTS;

  FUNCTION COEFICIENT (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_RESULTAT VARCHAR2) RETURN COEFICIENTS%ROWTYPE IS
  BEGIN
    DECLARE
      RETORN COEFICIENTS%ROWTYPE;
    BEGIN
      BEGIN
        SELECT *
        INTO RETORN
        FROM COEFICIENTS C
        WHERE C.MODALITAT = V_MODALITAT
          AND C.PARTIT = V_PARTIT
          AND C.RESULTAT = V_RESULTAT;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'COEFICIENTS ' || V_MODALITAT || ',' || V_PARTIT || ',' || V_RESULTAT, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'COEFICIENTS ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END COEFICIENT;
  


--
-- APOSTES
--
  FUNCTION ALTA_APOSTA (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_APOSTA VARCHAR2, V_IMPORT FLOAT) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
      V_ID NUMBER;
    BEGIN
      RETORN := ALTA_APOSTA (V_MODALITAT, V_PARTIT, V_APOSTA, V_IMPORT, V_ID);
      RETURN RETORN;
    END;
  END ALTA_APOSTA;
  
  FUNCTION ALTA_APOSTA (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_APOSTA VARCHAR2, V_IMPORT FLOAT, V_ID IN OUT NUMBER) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        INSERT INTO APOSTES
        VALUES (0, PKG_USUARIS.NOM_USUARI, V_MODALITAT, V_PARTIT, V_APOSTA, V_IMPORT, '0', '0')
        RETURNING ID INTO V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'APOSTA ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'APOSTA ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END ALTA_APOSTA;
          
  FUNCTION BAIXA_APOSTA (V_ID NUMBER) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        DELETE FROM APOSTES A
        WHERE A.ID = V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'APOSTA ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'APOSTA ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END BAIXA_APOSTA;
  
  FUNCTION MODIFICA_APOSTA (V_ID NUMBER, V_APOSTA VARCHAR2, V_IMPORT FLOAT) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        UPDATE APOSTES A 
        SET A.APOSTA = V_APOSTA,
            A.IMPORT = V_IMPORT
        WHERE A.ID = V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'APOSTA ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'APOSTA ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END MODIFICA_APOSTA;
  
  FUNCTION LLISTA_APOSTES RETURN SYS_REFCURSOR IS
  BEGIN
    DECLARE
      RETORN SYS_REFCURSOR;
    BEGIN
      BEGIN
        OPEN RETORN FOR
        SELECT *
        FROM APOSTES;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'APOSTA', '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'APOSTA ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END LLISTA_APOSTES;

  FUNCTION APOSTA (V_ID NUMBER) RETURN APOSTES%ROWTYPE IS
  BEGIN
    DECLARE
      RETORN APOSTES%ROWTYPE;
    BEGIN
      BEGIN
        SELECT *
        INTO RETORN
        FROM APOSTES A
        WHERE A.ID = V_ID;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'APOSTA ' || V_ID, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'APOSTA ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END APOSTA;
  
  FUNCTION CERCA_APOSTA (V_MODALITAT NUMBER, V_PARTIT NUMBER) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        SELECT MIN(A.ID)
        INTO RETORN
        FROM APOSTES A 
        WHERE A.MODALITAT = V_MODALITAT
          AND A.PARTIT = V_PARTIT
          AND A.NOM_USUARI = PKG_USUARIS.NOM_USUARI;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'CERCA', 'APOSTA ' || V_MODALITAT || V_PARTIT, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'CERCA', 'APOSTA ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END CERCA_APOSTA;



--
-- RESULTATS
--
  FUNCTION ALTA_RESULTAT (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_RESULTAT VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        INSERT INTO RESULTATS
        VALUES (V_MODALITAT, V_PARTIT, V_RESULTAT);
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'RESULTAT ' || V_MODALITAT || ',' || V_PARTIT, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'INSERT', 'RESULTAT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END ALTA_RESULTAT;
          
  FUNCTION BAIXA_RESULTAT (V_MODALITAT NUMBER, V_PARTIT NUMBER) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        DELETE FROM RESULTATS R
        WHERE R.MODALITAT = V_MODALITAT
          AND R.PARTIT = V_PARTIT;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'RESULTAT ' || V_MODALITAT || ',' || V_PARTIT, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'DELETE', 'RESULTAT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END BAIXA_RESULTAT;
  
  FUNCTION MODIFICA_RESULTAT (V_MODALITAT NUMBER, V_PARTIT NUMBER, V_RESULTAT VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      BEGIN
        UPDATE RESULTATS R 
        SET R.RESULTAT = V_RESULTAT
        WHERE R.MODALITAT = V_MODALITAT
          AND R.PARTIT = V_PARTIT;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'RESULTAT ' || V_MODALITAT || ',' || V_PARTIT, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          RETORN := SQLCODE;
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'UPDATE', 'RESULTAT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END MODIFICA_RESULTAT;
  
  FUNCTION LLISTA_RESULTATS RETURN SYS_REFCURSOR IS
  BEGIN
    DECLARE
      RETORN SYS_REFCURSOR;
    BEGIN
      BEGIN
        OPEN RETORN FOR
        SELECT *
        FROM RESULTATS;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'RESULTAT', '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'RESULTAT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END LLISTA_RESULTATS;

  FUNCTION RESULTAT (V_MODALITAT NUMBER, V_PARTIT NUMBER) RETURN RESULTATS%ROWTYPE IS
  BEGIN
    DECLARE
      RETORN RESULTATS%ROWTYPE;
    BEGIN
      BEGIN
        SELECT *
        INTO RETORN
        FROM RESULTATS R
        WHERE R.MODALITAT = V_MODALITAT
          AND R.PARTIT = V_PARTIT;
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'RESULTAT ' || V_MODALITAT || ',' || V_PARTIT, '0');
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT', 'RESULTAT ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END RESULTAT;
  


--
-- RESOLDRE_APOSTES
--
  PROCEDURE RESOLDRE_APOSTES IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
    END;
  END RESOLDRE_APOSTES;

END PKG_APOSTES;
/
