-- ********************
-- PKG_UTIL
-- ********************
CREATE OR REPLACE PACKAGE PKG_IMPORT AS

-- ********************
-- IMPORT_DW
  FUNCTION IMPORT_DATAWAREHOUSE RETURN BOOLEAN;
  
-- ********************
-- EXECUCIO
-- ********************
-- EXEMPLE D'EXECUCIO:
--
--DECLARE
-- B BOOLEAN;
--BEGIN
-- B:=PKG_IMPORT.IMPORT_DATAWAREHOUSE;
-- IF B THEN
--  DBMS_OUTPUT.PUT_LINE('IMPORT DATA WAREHOUSE OK');
-- ELSE
--  DBMS_OUTPUT.PUT_LINE('IMPORT DATA WAREHOUSE ERROR');
-- END IF;
-- EXCEPTION WHEN OTHERS THEN
--  DBMS_OUTPUT.PUT_LINE(SQLERRM);
--END;
--

-- ********************
-- RUTINES D'IMPORTACIO DE DIMENSIONS
--
-- IMPORT_D_ZONES
--
  FUNCTION IMPORT_D_ZONES RETURN BOOLEAN;

--
-- IMPORT_D_USUARIS
--
  FUNCTION IMPORT_D_USUARIS RETURN BOOLEAN;

--
-- IMPORT_D_DATES
--
  FUNCTION IMPORT_D_DATES RETURN BOOLEAN;

--
-- IMPORT_D_SEXES
--
  FUNCTION IMPORT_D_SEXES RETURN BOOLEAN;

--
-- IMPORT_D_EDATS
--
  FUNCTION IMPORT_D_EDATS RETURN BOOLEAN;

--
-- IMPORT_D_TEMPORADES
--
  FUNCTION IMPORT_D_TEMPORADES RETURN BOOLEAN;

--
-- IMPORT_D_COMPETICIONS
--
  FUNCTION IMPORT_D_COMPETICIONS RETURN BOOLEAN;

--
-- IMPORT_D_MODALITATS
--
  FUNCTION IMPORT_D_MODALITATS RETURN BOOLEAN;

--
-- IMPORT_D_EQUIPS
--
  FUNCTION IMPORT_D_EQUIPS RETURN BOOLEAN;


-- ********************
-- FUNCIONS DE CERCA DE CODIS USANT LES CLAUS ALTERNATIVES
--
  FUNCTION CERCA_ZONA(V_ALT VARCHAR2) RETURN NUMBER;
  FUNCTION CERCA_USUARI(V_ALT VARCHAR2) RETURN NUMBER;
  FUNCTION CERCA_DATA(V_ALT DATE) RETURN NUMBER;
  FUNCTION CERCA_SEXE(V_ALT VARCHAR2) RETURN NUMBER;
  FUNCTION CERCA_EDAT(V_ALT DATE) RETURN NUMBER;
  FUNCTION CERCA_TEMPORADA(V_ALT VARCHAR2) RETURN NUMBER;
  FUNCTION CERCA_COMPETICIO(V_ALT VARCHAR2) RETURN NUMBER;
  FUNCTION CERCA_MODALITAT(V_ALT VARCHAR2) RETURN NUMBER;
  FUNCTION CERCA_EQUIP(V_ALT VARCHAR2) RETURN NUMBER;

-- ********************
-- RUTINES D'IMPORTACIO DE FETS
--
-- IMPORT_F_APOSTES
--
  FUNCTION IMPORT_F_APOSTES RETURN BOOLEAN;

END PKG_IMPORT;
/

CREATE OR REPLACE PACKAGE BODY PKG_IMPORT AS


-- ********************
-- DEBUG
--
  PROCEDURE DEBUG(INFO VARCHAR2, RESULTAT BOOLEAN) IS
  BEGIN
    DECLARE
      B VARCHAR2(10);
    BEGIN
      IF (RESULTAT) THEN
        B := 'CERT';
      ELSE
        B := ' *** FALS ***';
      END IF;
      DBMS_OUTPUT.PUT_LINE(INFO || ' -> ' || B);
    END;
  END;

-- ********************
-- TRUNCATE_DATAWAREHOUSE
-- 
  FUNCTION TRUNCATE_DATAWAREHOUSE RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
/*
      BEGIN
        TRUNCATE TABLE F_APOSTES;
        TRUNCATE TABLE D_MODALITATS;
        TRUNCATE TABLE D_USUARIS;
        TRUNCATE TABLE D_EQUIPS;
        TRUNCATE TABLE D_EDATS;
        TRUNCATE TABLE D_SEXES;
        TRUNCATE TABLE D_DATES;
        TRUNCATE TABLE D_TEMPORADES;
        TRUNCATE TABLE D_COMPETICIONS;
        TRUNCATE TABLE D_ZONES;

        COMMIT;
      EXCEPTION WHEN OTHERS THEN
        ROLLBACK;
      END;
*/
      RETURN RETORN;
    END;
  END TRUNCATE_DATAWAREHOUSE;

-- ********************
-- IMPORT_DATAWAREHOUSE
--
  FUNCTION IMPORT_DATAWAREHOUSE RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;

      PARCIAL := IMPORT_D_ZONES;
      RETORN := RETORN AND PARCIAL;
      DEBUG('IMPORT_DATAWAREHOUSE: IMPORT_D_ZONES ', PARCIAL);
      
      PARCIAL := IMPORT_D_USUARIS;
      RETORN := RETORN AND PARCIAL;
      DEBUG('IMPORT_DATAWAREHOUSE: IMPORT_D_USUARIS ', PARCIAL);
      
      PARCIAL := IMPORT_D_DATES;
      RETORN := RETORN AND PARCIAL;
      DEBUG('IMPORT_DATAWAREHOUSE: IMPORT_D_DATES ', PARCIAL);
      
      PARCIAL := IMPORT_D_SEXES;
      RETORN := RETORN AND PARCIAL;
      DEBUG('IMPORT_DATAWAREHOUSE: IMPORT_D_SEXES ', PARCIAL);
      
      PARCIAL := IMPORT_D_EDATS;
      RETORN := RETORN AND PARCIAL;
      DEBUG('IMPORT_DATAWAREHOUSE: IMPORT_D_EDATS ', PARCIAL);
      
      PARCIAL := IMPORT_D_TEMPORADES;
      RETORN := RETORN AND PARCIAL;
      DEBUG('IMPORT_DATAWAREHOUSE: IMPORT_D_TEMPORADES ', PARCIAL);
      
      PARCIAL := IMPORT_D_COMPETICIONS;
      RETORN := RETORN AND PARCIAL;
      DEBUG('IMPORT_DATAWAREHOUSE: IMPORT_D_COMPETICIONS ', PARCIAL);
      
      PARCIAL := IMPORT_D_MODALITATS;
      RETORN := RETORN AND PARCIAL;
      DEBUG('IMPORT_DATAWAREHOUSE: IMPORT_D_MODALITATS ', PARCIAL);
      
      PARCIAL := IMPORT_D_EQUIPS;
      RETORN := RETORN AND PARCIAL;
      DEBUG('IMPORT_DATAWAREHOUSE: IMPORT_D_EQUIPS ', PARCIAL);
      
      --PARCIAL := IMPORT_F_APOSTES;
      PARCIAL:=true;
      RETORN := RETORN AND PARCIAL;
      DEBUG('IMPORT_DATAWAREHOUSE: IMPORT_F_APOSTES ', PARCIAL);
      
      IF RETORN THEN
        COMMIT;
      ELSE
        ROLLBACK;
      END IF;
      
      RETURN RETORN;
    END;
  END IMPORT_DATAWAREHOUSE;

-- ********************
-- IMPORT_D_ZONES
--
  FUNCTION IMPORT_D_ZONES RETURN BOOLEAN IS
  BEGIN
    DECLARE
      RETORN BOOLEAN;
    BEGIN
      RETORN := TRUE;
      BEGIN
        INSERT INTO D_ZONES
        SELECT SEQ_SGAF_DW.NEXTVAL, Z.CODI, Z.NOM
        FROM SGAF.ZONES Z;
        DEBUG('IMPORT_D_ZONES: IMPORTACIO OK', TRUE);
      EXCEPTION WHEN OTHERS THEN
        RETORN := FALSE;
        DEBUG('IMPORT_D_ZONES: IMPORTACIO ERROR ' || SQLERRM, FALSE);
      END;
      RETURN RETORN;
    END;
  END IMPORT_D_ZONES;

-- ********************
-- IMPORT_D_USUARIS
--
  FUNCTION IMPORT_D_USUARIS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      RETORN BOOLEAN;
    BEGIN
      RETORN := TRUE;
      BEGIN
        INSERT INTO D_USUARIS
        SELECT SEQ_SGAF_DW.NEXTVAL, U.NOM_USUARI
        FROM SGAF.USUARIS U;
        DEBUG('IMPORT_D_USUARIS: IMPORTACIO OK', TRUE);
      EXCEPTION WHEN OTHERS THEN
        RETORN := FALSE;
        DEBUG('IMPORT_D_USUARIS: IMPORTACIO ERROR ' || SQLERRM, FALSE);
      END;
      RETURN RETORN;
    END;
  END IMPORT_D_USUARIS;

-- ********************
-- IMPORT_D_DATES
--
  FUNCTION IMPORT_D_DATES RETURN BOOLEAN IS
  BEGIN
    DECLARE
      RETORN BOOLEAN;
    BEGIN
      RETORN := TRUE;
      BEGIN
        INSERT INTO D_DATES
        SELECT SEQ_SGAF_DW.NEXTVAL, TO_NUMBER( TO_CHAR(O_P.DATA, 'YYYY') ), TO_NUMBER( TO_CHAR(O_P.DATA, 'IW') ), TO_NUMBER( TO_CHAR(O_P.DATA, 'MM') ), TO_CHAR(O_P.DATA, 'MONTH'), TO_NUMBER( TO_CHAR(O_P.DATA, 'DD') ), TO_NUMBER( TO_CHAR(O_P.DATA, 'HH24') ), TO_NUMBER( TO_CHAR(O_P.DATA, 'MI') ), TO_NUMBER( TO_CHAR(O_P.DATA, 'SS') ), TO_CHAR(O_P.DATA, 'DAY')
        FROM (SELECT DISTINCT P.DATA 
              FROM SGAF.PARTITS P
             ) O_P;
        DEBUG('IMPORT_D_DATES: IMPORTACIO OK', TRUE);
      EXCEPTION WHEN OTHERS THEN
        RETORN := FALSE;
        DEBUG('IMPORT_D_DATES: IMPORTACIO ERROR ' || SQLERRM, FALSE);
      END;
      RETURN RETORN;
    END;
  END IMPORT_D_DATES;

-- ********************
-- IMPORT_D_SEXES
--
  FUNCTION IMPORT_D_SEXES RETURN BOOLEAN IS
  BEGIN
    DECLARE
      RETORN BOOLEAN;
    BEGIN
      RETORN := TRUE;
      BEGIN
        INSERT INTO D_SEXES
        SELECT SEQ_SGAF_DW.NEXTVAL, U_L.SEXE
        FROM ( SELECT DISTINCT SEXE
               FROM SGAF.USUARIS_LOPD) U_L;
        DEBUG('IMPORT_D_SEXES: IMPORTACIO OK', TRUE);
      EXCEPTION WHEN OTHERS THEN
        RETORN := FALSE;
        DEBUG('IMPORT_D_SEXES: IMPORTACIO ERROR ' || SQLERRM, FALSE);
      END;
      RETURN RETORN;
    END;
  END IMPORT_D_SEXES;

-- ********************
-- IMPORT_D_EDATS
--
  FUNCTION IMPORT_D_EDATS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      RETORN BOOLEAN;
    BEGIN
      RETORN := TRUE;
      BEGIN
        INSERT INTO D_EDATS
        SELECT SEQ_SGAF_DW.NEXTVAL, U_L.EDAT
        FROM ( SELECT DISTINCT TRUNC((SYSDATE - NAIXEMENT)/365) EDAT
               FROM SGAF.USUARIS_LOPD) U_L;
        DEBUG('IMPORT_D_EDATS: IMPORTACIO OK', TRUE);
      EXCEPTION WHEN OTHERS THEN
        RETORN := FALSE;
        DEBUG('IMPORT_D_EDATS: IMPORTACIO ERROR ' || SQLERRM, FALSE);
      END;
      RETURN RETORN;
    END;
  END IMPORT_D_EDATS;

-- ********************
-- IMPORT_D_TEMPORADES
--
  FUNCTION IMPORT_D_TEMPORADES RETURN BOOLEAN IS
  BEGIN
    DECLARE
      RETORN BOOLEAN;
    BEGIN
      RETORN := TRUE;
      BEGIN
        INSERT INTO D_TEMPORADES
        SELECT SEQ_SGAF_DW.NEXTVAL, T.AAAA
        FROM ( SELECT DISTINCT AAAA
               FROM SGAF.TEMPORADES) T;
        DEBUG('IMPORT_D_TEMPORADES: IMPORTACIO OK', TRUE);
      EXCEPTION WHEN OTHERS THEN
        RETORN := FALSE;
        DEBUG('IMPORT_D_TEMPORADES: IMPORTACIO ERROR ' || SQLERRM, FALSE);
      END;
      RETURN RETORN;
    END;
  END IMPORT_D_TEMPORADES;

-- ********************
-- IMPORT_D_COMPETICIONS
--
  FUNCTION IMPORT_D_COMPETICIONS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      RETORN BOOLEAN;
    BEGIN
      RETORN := TRUE;
      BEGIN
        INSERT INTO D_COMPETICIONS
        SELECT SEQ_SGAF_DW.NEXTVAL, C.CODI, C.DESCRIPCIO
        FROM SGAF.COMPETICIONS C;
        DEBUG('IMPORT_D_COMPETICIONS: IMPORTACIO OK', TRUE);
      EXCEPTION WHEN OTHERS THEN
        RETORN := FALSE;
        DEBUG('IMPORT_D_COMPETICIONS: IMPORTACIO ERROR ' || SQLERRM, FALSE);
      END;
      RETURN RETORN;
    END;
  END IMPORT_D_COMPETICIONS;

-- ********************
-- IMPORT_D_MODALITATS
--
  FUNCTION IMPORT_D_MODALITATS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      RETORN BOOLEAN;
    BEGIN
      RETORN := TRUE;
      BEGIN
        INSERT INTO D_MODALITATS
        SELECT SEQ_SGAF_DW.NEXTVAL, M.DESCRIPCIO
        FROM SGAF.MODALITATS M;
        DEBUG('IMPORT_D_MODALITATS: IMPORTACIO OK', TRUE);
      EXCEPTION WHEN OTHERS THEN
        RETORN := FALSE;
        DEBUG('IMPORT_D_MODALITATS: IMPORTACIO ERROR ' || SQLERRM, FALSE);
      END;
      RETURN RETORN;
    END;
  END IMPORT_D_MODALITATS;

-- ********************
-- IMPORT_D_EQUIPS
--
  FUNCTION IMPORT_D_EQUIPS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      RETORN BOOLEAN;
    BEGIN
      RETORN := TRUE;
      BEGIN
        INSERT INTO D_EQUIPS
        SELECT SEQ_SGAF_DW.NEXTVAL, E.CODI, E.NOM
        FROM SGAF.EQUIPS E;
        DEBUG('IMPORT_D_EQUIPS: IMPORTACIO OK', TRUE);
      EXCEPTION WHEN OTHERS THEN
        RETORN := FALSE;
        DEBUG('IMPORT_D_EQUIPS: IMPORTACIO ERROR ' || SQLERRM, FALSE);
      END;
      RETURN RETORN;
    END;
  END IMPORT_D_EQUIPS;

-- ********************
-- FUNCIONS DE CERCA DE CODIS USANT LES CLAUS ALTERNATIVES
--
  FUNCTION CERCA_ZONA(V_ALT VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      SELECT ID
      INTO RETORN
      FROM D_ZONES
      WHERE CODI = V_ALT;
      RETURN RETORN;
    END;
  END CERCA_ZONA;

  FUNCTION CERCA_USUARI(V_ALT VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      SELECT ID
      INTO RETORN
      FROM D_USUARIS
      WHERE NOM_USUARI = V_ALT;
      RETURN RETORN;
    END;
  END CERCA_USUARI;

  FUNCTION CERCA_DATA(V_ALT DATE) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      SELECT ID
      INTO RETORN
      FROM D_DATES
      WHERE AAAA = TO_NUMBER( TO_CHAR(V_ALT, 'YYYY') )
        AND MES = TO_NUMBER( TO_CHAR(V_ALT, 'MM') ) 
        AND DIA = TO_NUMBER( TO_CHAR(V_ALT, 'DD') )
        AND HORA = TO_NUMBER( TO_CHAR(V_ALT, 'HH24') )
        AND MINUT = TO_NUMBER( TO_CHAR(V_ALT, 'MI') )
        AND SEGON = TO_NUMBER( TO_CHAR(V_ALT, 'SS') );
      RETURN RETORN;
    END;
  END CERCA_DATA;

  FUNCTION CERCA_SEXE(V_ALT VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      SELECT ID
      INTO RETORN
      FROM D_SEXES
      WHERE SEXE = V_ALT;
      RETURN RETORN;
    END;
  END CERCA_SEXE;

  FUNCTION CERCA_EDAT(V_ALT DATE) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      SELECT ID
      INTO RETORN
      FROM D_EDATS
      WHERE EDAT = TRUNC((SYSDATE - V_ALT)/365);
      RETURN RETORN;
    END;
  END CERCA_EDAT;

  FUNCTION CERCA_TEMPORADA(V_ALT VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      SELECT ID
      INTO RETORN
      FROM D_TEMPORADES
      WHERE AAAA = V_ALT;
      RETURN RETORN;
    END;
  END CERCA_TEMPORADA;

  FUNCTION CERCA_COMPETICIO(V_ALT VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      SELECT ID
      INTO RETORN
      FROM D_COMPETICIONS
      WHERE CODI = V_ALT;
      RETURN RETORN;
    END;
  END CERCA_COMPETICIO;

  FUNCTION CERCA_MODALITAT(V_ALT VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      SELECT ID
      INTO RETORN
      FROM D_MODALITATS
      WHERE DESCRIPCIO = V_ALT;
      RETURN RETORN;
    END;
  END CERCA_MODALITAT;

  FUNCTION CERCA_EQUIP(V_ALT VARCHAR2) RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      SELECT ID
      INTO RETORN
      FROM D_EQUIPS
      WHERE CODI = V_ALT;
      RETURN RETORN;
    END;
  END CERCA_EQUIP;


-- ********************
-- IMPORT_F_APOSTES
--
  FUNCTION IMPORT_F_APOSTES RETURN BOOLEAN IS
  BEGIN
    DECLARE
      RETORN BOOLEAN;
    BEGIN
      RETORN := TRUE;

      INSERT INTO F_APOSTES
      SELECT SEQ_SGAF_DW.NEXTVAL,
             CERCA_USUARI(A.NOM_USUARI),
             CERCA_DATA(P.DATA),
             CERCA_COMPETICIO(P.COMPETICIO),
             CERCA_TEMPORADA(P.AAAA),
             CERCA_MODALITAT(M.DESCRIPCIO),
             CERCA_EQUIP(P.EQUIP1),
             CERCA_EQUIP(P.EQUIP2),
             CERCA_EDAT(U_L.NAIXEMENT),
             CERCA_SEXE(U_L.SEXE),
             CERCA_ZONA(C.ZONA),
             CERCA_ZONA(U_L.NACIONALITAT),
             CERCA_ZONA(E1.PAIS),
             CERCA_ZONA(E2.PAIS),
             A.IMPORT, 
             A.GUANYADORA, 
             DECODE(A.GUANYADORA, '1', CF.VALOR*A.IMPORT, 0)
      FROM SGAF.APOSTES A, SGAF.PARTITS P, SGAF.MODALITATS M, SGAF.USUARIS_LOPD U_L, SGAF.COMPETICIONS C, SGAF.EQUIPS E1, SGAF.EQUIPS E2, SGAF.COEFICIENTS CF
      WHERE A.PARTIT = P.ID
        AND A.MODALITAT = M.ID
        AND A.NOM_USUARI = U_L.NOM_USUARI
        AND P.COMPETICIO = C.CODI
        AND P.EQUIP1 = E1.CODI
        AND P.EQUIP2 = E2.CODI
        AND A.PARTIT = CF.PARTIT
        AND A.MODALITAT = CF.MODALITAT
        AND A.APOSTA = CF.RESULTAT;

      RETURN RETORN;
    END;
  END IMPORT_F_APOSTES;

END PKG_IMPORT;
/
