-- ********************
-- PKG_UTIL
-- ********************
CREATE OR REPLACE PACKAGE PKG_UTIL AS

-- ********************
-- AFEGIR_LOG
-- AFEGEIX UN REGISTRE AL LOG DEL SISTEMA
--
  PROCEDURE AFEGIR_LOG (USUARI VARCHAR2, ACCIO VARCHAR2, INFO VARCHAR2, RESULTAT VARCHAR2);
--
-- LLISTA_LOGS
-- RETORNA LA LLISTA DE USUARIS
-- S'HA D'EXECUTAR AIX:
--   sql> variable c refcursor
--   sql> exec :c := pkg_util.llista_logs();
--   sql> print c;
--
-- REFERENCIA:
--   http://dba.stackexchange.com/questions/18256/return-many-rows-on-a-plsql-oracle10g
--
  FUNCTION LLISTA_LOGS RETURN SYS_REFCURSOR;

-- ********************
-- TEST
-- ********************
-- EXEMPLE D'EXECUCIO:
--
--DECLARE
-- B BOOLEAN;
--BEGIN
-- B:=PKG_UTIL.TEST_ZONES;
-- IF B THEN
--  DBMS_OUTPUT.PUT_LINE('FINAL OK');
-- ELSE
--  DBMS_OUTPUT.PUT_LINE('FINAL ERROR');
-- END IF;
-- EXCEPTION WHEN OTHERS THEN
--  DBMS_OUTPUT.PUT_LINE(SQLERRM);
--END;
--

-- ********************
-- RUTINES DE TEST DEL PACKAGE: PKG_COMPETICIONS
--
-- TEST ZONES
--
  FUNCTION TEST_ZONES RETURN BOOLEAN;
--
-- TEST COMPETICIONS
--
  FUNCTION TEST_COMPETICIONS RETURN BOOLEAN;
--
-- TEST TEMPORADES
--
  FUNCTION TEST_TEMPORADES RETURN BOOLEAN;
--
-- TEST EQUIPS
--
  FUNCTION TEST_EQUIPS RETURN BOOLEAN;
--
-- TEST JUGADORS
--
  FUNCTION TEST_JUGADORS RETURN BOOLEAN;
--
-- TEST PLANTILLES
--
  FUNCTION TEST_PLANTILLES RETURN BOOLEAN;


-- ********************
-- RUTINES DE TEST DEL PACKAGE: PKG_PARTITS
--
-- TEST PARTITS
--
  FUNCTION TEST_PARTITS RETURN BOOLEAN;
--
-- TEST TIPUS_GOL
--
  FUNCTION TEST_TIPUS_GOL RETURN BOOLEAN;
--
-- TEST GOLS
--
  FUNCTION TEST_GOLS RETURN BOOLEAN;


-- ********************
-- RUTINES DE TEST DEL PACKAGE: PKG_USUARIS
--
-- TEST USUARIS
--
  FUNCTION TEST_USUARIS RETURN BOOLEAN;


-- ********************
-- RUTINES DE TEST DEL PACKAGE: PKG_APOSTES
--
-- TEST MODALITATS
--
  FUNCTION TEST_MODALITATS RETURN BOOLEAN;
--
-- TEST COEFICIENTS
--
  FUNCTION TEST_COEFICIENTS RETURN BOOLEAN;
--
-- TEST APOSTES
--
  FUNCTION TEST_APOSTES RETURN BOOLEAN;
--
-- TEST RESULTATS
--
  FUNCTION TEST_RESULTATS RETURN BOOLEAN;


-- ********************
-- DADES INICIALS
-- INSERCIO DE LES DADES INICIALS DEL SISTEMA PER PODER FER PROVES
--
  FUNCTION DADES_INICIALS RETURN NUMBER;
  
END PKG_UTIL;
/

-- ********************
-- PKG_UTIL BODY
-- ********************
CREATE OR REPLACE PACKAGE BODY PKG_UTIL AS

-- ********************
-- AFEGIR_LOG
-- AFEGEIX UN REGISTRE AL LOG DEL SISTEMA
--
  PROCEDURE AFEGIR_LOG (USUARI VARCHAR2, ACCIO VARCHAR2, INFO VARCHAR2, RESULTAT VARCHAR2) IS
  PRAGMA AUTONOMOUS_TRANSACTION;
  BEGIN
    INSERT INTO LOG_ACCIONS
    VALUES (USUARI, ACCIO, INFO, SYSDATE(), RESULTAT);
    COMMIT;
  END AFEGIR_LOG;

-- ********************
-- LLISTA_LOGS
-- LLISTA EL LOG DEL SISTEMA
--
  FUNCTION LLISTA_LOGS RETURN SYS_REFCURSOR IS
  BEGIN
    DECLARE
      RETORN SYS_REFCURSOR;
    BEGIN
      BEGIN
        PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'LOG', '0');
        OPEN RETORN FOR
        SELECT *
        FROM LOG_ACCIONS;
      EXCEPTION 
        WHEN OTHERS THEN
          PKG_UTIL.AFEGIR_LOG (PKG_USUARIS.NOM_USUARI, 'SELECT ALL', 'LOG ' || SQLERRM, SQLCODE);
      END;
      RETURN RETORN;
    END;
  END LLISTA_LOGS;


-- ********************
-- DEBUG
--
  PROCEDURE DEBUG(INFO VARCHAR2, RESULTAT BOOLEAN) IS
  BEGIN
    DECLARE
      B VARCHAR2(10);
    BEGIN
      IF (RESULTAT) THEN
        B := 'CERT';
      ELSE
        B := ' *** FALS ***';
      END IF;
      DBMS_OUTPUT.PUT_LINE(INFO || ' -> ' || B);
    END;
  END;


-- ********************
-- TEST_ZONES
--
  FUNCTION TEST_ZONES RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      Z ZONES%ROWTYPE;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;
      
      -- AFEGIM UNA ZONA
      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z1', 'PRIMERA ZONA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: ALTA_ZONA Z1 ', PARCIAL);
      
      -- LA CONSULTEM
      Z := PKG_COMPETICIONS.ZONA('Z1');
      PARCIAL := (Z.NOM = 'PRIMERA ZONA');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: ZONA Z1 ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: BAIXA_ZONA Z1 ', PARCIAL);

      Z := PKG_COMPETICIONS.ZONA('Z1');
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (Z IS NULL) THEN 
      IF (NVL(Z.NOM,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
			RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: ZONA Z1 NO DISPONIBLE ', PARCIAL);



      -- AFEGIM DUES ZONES
      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z2', 'SEGONA ZONA');
      PARCIAL := (UNITARI = 0);
     RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: ALTA_ZONA Z2 ', PARCIAL);

      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z3', 'TERCERA ZONA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: ALTA_ZONA Z3 ', PARCIAL);
      
      -- LES CONSULTEM
      Z := PKG_COMPETICIONS.ZONA('Z2');
      PARCIAL := (Z.NOM = 'SEGONA ZONA');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: ZONA Z2 ', PARCIAL);

      Z := PKG_COMPETICIONS.ZONA('Z3');
      PARCIAL := (Z.NOM = 'TERCERA ZONA');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: ZONA Z3 ', PARCIAL);
      
      -- LES ESBORREM
      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z2');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: BAIXA_ZONA Z2 ', PARCIAL);

      Z := PKG_COMPETICIONS.ZONA('Z2');
      IF (NVL(Z.NOM,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: ZONA Z2 NO DISPONIBLE ', PARCIAL);

      Z := PKG_COMPETICIONS.ZONA('Z3');
      PARCIAL := (Z.NOM = 'TERCERA ZONA');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: ZONA Z3 ', PARCIAL);

      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z3');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: BAIXA_ZONA Z3 ', PARCIAL);

      Z := PKG_COMPETICIONS.ZONA('Z3');
      IF (NVL(Z.NOM,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_ZONES: ZONA Z3 NO DISPONIBLE ', PARCIAL);

      RETURN RETORN;
    END;
  END TEST_ZONES;


-- ********************
-- TEST_COMPETICIONS
--
  FUNCTION TEST_COMPETICIONS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      Z ZONES%ROWTYPE;
      C COMPETICIONS%ROWTYPE;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;
      
      -- AFEGIM UNA ZONA
      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z1', 'PRIMERA ZONA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: ALTA_ZONA Z1 ', PARCIAL);
      
      -- AFEGIM UNA COMPETICIO
      UNITARI := PKG_COMPETICIONS.ALTA_COMPETICIO('C1', 'PRIMERA COMPETICIO', 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: ALTA_COMPETICIO C1 ', PARCIAL);
      
      -- LA CONSULTEM
      C := PKG_COMPETICIONS.COMPETICIO('C1');
      PARCIAL := (C.DESCRIPCIO = 'PRIMERA COMPETICIO');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: COMPETICIO C1 ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_COMPETICIONS.BAIXA_COMPETICIO('C1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: BAIXA_COMPETICIO C1 ', PARCIAL);

      C := PKG_COMPETICIONS.COMPETICIO('C1');
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (C IS NULL) THEN 
      IF (NVL(C.DESCRIPCIO,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: COMPETICIO C1 NO DISPONIBLE', PARCIAL);



      -- AFEGIM DUES COMPETICIONS
      UNITARI := PKG_COMPETICIONS.ALTA_COMPETICIO('C2', 'SEGONA COMPETICIO', 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: ALTA_COMPETICIO C2', PARCIAL);

      UNITARI := PKG_COMPETICIONS.ALTA_COMPETICIO('C3', 'TERCERA COMPETICIO', 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: ALTA_COMPETICIO C3', PARCIAL);
      
      -- LES CONSULTEM
      C := PKG_COMPETICIONS.COMPETICIO('C2');
      PARCIAL := (C.DESCRIPCIO = 'SEGONA COMPETICIO');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: COMPETICIO C2 ', PARCIAL);

      C := PKG_COMPETICIONS.COMPETICIO('C3');
      PARCIAL := (C.DESCRIPCIO = 'TERCERA COMPETICIO');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: COMPETICIO C3 ', PARCIAL);
      
      -- LES ESBORREM
      UNITARI := PKG_COMPETICIONS.BAIXA_COMPETICIO('C2');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: BAIXA_COMPETICIO C2 ', PARCIAL);

      C := PKG_COMPETICIONS.COMPETICIO('C2');
      IF (NVL(C.DESCRIPCIO,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: COMPETICIO C2 NO DISPONIBLE', PARCIAL);

      C := PKG_COMPETICIONS.COMPETICIO('C3');
      PARCIAL := (C.DESCRIPCIO = 'TERCERA COMPETICIO');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: COMPETICIO C3 ', PARCIAL);

      UNITARI := PKG_COMPETICIONS.BAIXA_COMPETICIO('C3');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: BAIXA_COMPETICIO C3 ', PARCIAL);

      C := PKG_COMPETICIONS.COMPETICIO('C3');
      IF (NVL(C.DESCRIPCIO,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: COMPETICIO C3 NO DISPONIBLE', PARCIAL);

      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COMPETICIONS: BAIXA_ZONA Z1 ', PARCIAL);

      RETURN RETORN;
    END;
  END TEST_COMPETICIONS;


-- ********************
-- TEST_TEMPORADES
--
  FUNCTION TEST_TEMPORADES RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      Z ZONES%ROWTYPE;
      C COMPETICIONS%ROWTYPE;
      T TEMPORADES%ROWTYPE;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;
      
      -- AFEGIM UNA ZONA
      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z1', 'PRIMERA ZONA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: ALTA_ZONA Z1', PARCIAL);
      
      -- AFEGIM UNA COMPETICIO
      UNITARI := PKG_COMPETICIONS.ALTA_COMPETICIO('C1', 'PRIMERA COMPETICIO', 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: ALTA_COMPETICIO C1', PARCIAL);
      
      -- AFEGIM UNA TEMPORADA
      UNITARI := PKG_COMPETICIONS.ALTA_TEMPORADA('C1', 2001, TO_DATE('01/01/2001', 'DD/MM/YYYY'), TO_DATE('31/12/2001', 'DD/MM/YYYY'));
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: ALTA_TEMPORADA C1-2001', PARCIAL);
      
      -- LA CONSULTEM
      T := PKG_COMPETICIONS.TEMPORADA('C1', 2001);
      PARCIAL := (T.INICI = TO_DATE('01/01/2001', 'DD/MM/YYYY'));
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: TEMPORADA C1-2001 ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_COMPETICIONS.BAIXA_TEMPORADA('C1', 2001);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: TEMPORADA C1-2001 ', PARCIAL);

      T := PKG_COMPETICIONS.TEMPORADA('C1', 2001);
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (C IS NULL) THEN 
      IF (NVL(T.COMPETICIO,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: TEMPORADA C1-2001 NO DISPONIBLE', PARCIAL);



      -- AFEGIM DUES COMPETICIONS
      UNITARI := PKG_COMPETICIONS.ALTA_TEMPORADA('C1', 2002, TO_DATE('02/02/2002', 'DD/MM/YYYY'), TO_DATE('31/12/2002', 'DD/MM/YYYY'));
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: ALTA_TEMPORADA C1-2002', PARCIAL);

      UNITARI := PKG_COMPETICIONS.ALTA_TEMPORADA('C1', 2003, TO_DATE('03/03/2003', 'DD/MM/YYYY'), TO_DATE('31/12/2003', 'DD/MM/YYYY'));
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: ALTA_TEMPORADA C1-2003', PARCIAL);
      
      -- LES CONSULTEM
      T := PKG_COMPETICIONS.TEMPORADA('C1', 2002);
      PARCIAL := (T.INICI = TO_DATE('02/02/2002', 'DD/MM/YYYY'));
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: TEMPORADA C1-2002 ', PARCIAL);

      T := PKG_COMPETICIONS.TEMPORADA('C1', 2003);
      PARCIAL := (T.INICI = TO_DATE('03/03/2003', 'DD/MM/YYYY'));
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: TEMPORADA C1-2003 ', PARCIAL);
      
      -- LES ESBORREM
      UNITARI := PKG_COMPETICIONS.BAIXA_TEMPORADA('C1', 2002);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: BAIXA_TEMPORADA C1-2002 ', PARCIAL);

      T := PKG_COMPETICIONS.TEMPORADA('C1', 2002);
      IF (NVL(T.COMPETICIO,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: TEMPORADA C1-2002 NO DISPONIBLE', PARCIAL);

      T := PKG_COMPETICIONS.TEMPORADA('C1', 2003);
      PARCIAL := (T.INICI = TO_DATE('03/03/2003', 'DD/MM/YYYY'));
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: TEMPORADA C1-2003 ', PARCIAL);

      UNITARI := PKG_COMPETICIONS.BAIXA_TEMPORADA('C1', 2003);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: BAIXA_TEMPORADA C1-2003 ', PARCIAL);

      T := PKG_COMPETICIONS.TEMPORADA('C1', 2003);
      IF (NVL(T.COMPETICIO,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: TEMPORADA C1-2003 NO DISPONIBLE', PARCIAL);

      UNITARI := PKG_COMPETICIONS.BAIXA_COMPETICIO('C1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: BAIXA_COMPETICIO C1 ', PARCIAL);

      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TEMPORADES: BAIXA_ZONA Z1 ', PARCIAL);

      RETURN RETORN;
    END;
  END TEST_TEMPORADES;


-- ********************
-- TEST_EQUIPS
--
  FUNCTION TEST_EQUIPS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      Z ZONES%ROWTYPE;
      E EQUIPS%ROWTYPE;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;
      
      -- AFEGIM UNA ZONA
      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z1', 'PRIMERA ZONA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: ALTA_ZONA Z1', PARCIAL);
      
      -- AFEGIM UN EQUIP
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ1', 'PRIMER EQUIP', TO_DATE('01/01/1899', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: ALTA_EQUIP EQ1', PARCIAL);
      
      -- EL CONSULTEM
      E := PKG_COMPETICIONS.EQUIP('EQ1');
      PARCIAL := (E.NOM = 'PRIMER EQUIP');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: EQUIP E1 ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: BAIXA_EQUIP EQ1 ', PARCIAL);

      E := PKG_COMPETICIONS.EQUIP('EQ1');
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (E IS NULL) THEN 
      IF (NVL(E.NOM,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: EQUIP EQ1 NO DISPONIBLE', PARCIAL);



      -- AFEGIM DOS EQUIPS
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ2', 'SEGON EQUIP', TO_DATE('01/01/1900', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: ALTA_EQUIP EQ2', PARCIAL);

      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ3', 'TERCER EQUIP', TO_DATE('01/01/1901', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: ALTA_EQUIP EQ3', PARCIAL);
      
      -- ELS CONSULTEM
      E := PKG_COMPETICIONS.EQUIP('EQ2');
      PARCIAL := (E.NOM = 'SEGON EQUIP');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: EQUIP EQ2 ', PARCIAL);

      E := PKG_COMPETICIONS.EQUIP('EQ3');
      PARCIAL := (E.NOM = 'TERCER EQUIP');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: EQUIP EQ3 ', PARCIAL);
      
      -- ELS ESBORREM
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ2');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: BAIXA_EQUIP EQ2 ', PARCIAL);

      E := PKG_COMPETICIONS.EQUIP('EQ2');
      IF (NVL(E.NOM,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: EQUIP C2 NO DISPONIBLE', PARCIAL);

      E := PKG_COMPETICIONS.EQUIP('EQ3');
      PARCIAL := (E.NOM = 'TERCER EQUIP');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: EQUIP EQ3 ', PARCIAL);

      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ3');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: BAIXA_EQUIP EQ3 ', PARCIAL);

      E := PKG_COMPETICIONS.EQUIP('EQ3');
      IF (NVL(E.NOM,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: EQUIP EQ3 NO DISPONIBLE', PARCIAL);

      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_EQUIPS: BAIXA_ZONA Z1 ', PARCIAL);

      RETURN RETORN;
    END;
  END TEST_EQUIPS;


-- ********************
-- TEST_JUGADORS
--
  FUNCTION TEST_JUGADORS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      Z ZONES%ROWTYPE;
      J JUGADORS%ROWTYPE;
      J1_ID NUMBER;
      J2_ID NUMBER;
      J3_ID NUMBER;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;
      J1_ID := 0;
      J2_ID := 0;
      J3_ID := 0;
      
      -- AFEGIM UNA ZONA
      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z1', 'PRIMERA ZONA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: ALTA_ZONA Z1', PARCIAL);
      
      -- AFEGIM UN JUGADOR
      UNITARI := PKG_COMPETICIONS.ALTA_JUGADOR('NOM1', 'COGNOM11', 'COGNOM21', TO_DATE('01/01/1899', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: ALTA_JUGADOR NOM1', PARCIAL);
      
      -- EL CONSULTEM
      J1_ID := PKG_COMPETICIONS.CERCA_JUGADOR('NOM1', 'COGNOM11', 'COGNOM21');
      J := PKG_COMPETICIONS.JUGADOR(J1_ID);
      PARCIAL := (J.NAIXEMENT = TO_DATE('01/01/1899', 'DD/MM/YYYY'));
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: JUGADOR NOM1 ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_COMPETICIONS.BAIXA_JUGADOR(J1_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: BAIXA_JUGADOR NOM1 ', PARCIAL);

      J := PKG_COMPETICIONS.JUGADOR(J1_ID);
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (J IS NULL) THEN 
      IF (NVL(J.NOM,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: JUGADOR NOM1 NO DISPONIBLE', PARCIAL);



      -- AFEGIM DOS EQUIPS
      UNITARI := PKG_COMPETICIONS.ALTA_JUGADOR('NOM2', 'COGNOM12', 'COGNOM22', TO_DATE('01/01/1900', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: ALTA_JUGADOR NOM2', PARCIAL);

      UNITARI := PKG_COMPETICIONS.ALTA_JUGADOR('NOM3', 'COGNOM13', 'COGNOM23', TO_DATE('01/01/1901', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: ALTA_JUGADOR NOM3', PARCIAL);
      
      -- ELS CONSULTEM
      J2_ID := PKG_COMPETICIONS.CERCA_JUGADOR('NOM2', 'COGNOM12', 'COGNOM22');
      J := PKG_COMPETICIONS.JUGADOR(J2_ID);
      PARCIAL := (J.NAIXEMENT = TO_DATE('01/01/1900', 'DD/MM/YYYY'));
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: JUGADOR NOM2 ', PARCIAL);

      J3_ID := PKG_COMPETICIONS.CERCA_JUGADOR('NOM3', 'COGNOM13', 'COGNOM23');
      J := PKG_COMPETICIONS.JUGADOR(J3_ID);
      PARCIAL := (J.NAIXEMENT = TO_DATE('01/01/1901', 'DD/MM/YYYY'));
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: JUGADOR NOM3 ', PARCIAL);
      
      -- ELS ESBORREM
      UNITARI := PKG_COMPETICIONS.BAIXA_JUGADOR(J2_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: BAIXA_JUGADOR NOM2 ', PARCIAL);

      J := PKG_COMPETICIONS.JUGADOR(J2_ID);
      IF (NVL(J.NOM,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: JUGADOR NOM2 NO DISPONIBLE', PARCIAL);

      J := PKG_COMPETICIONS.JUGADOR(J3_ID);
      PARCIAL := (J.NAIXEMENT = TO_DATE('01/01/1901', 'DD/MM/YYYY'));
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: JUGADOR NOM3 ', PARCIAL);

      UNITARI := PKG_COMPETICIONS.BAIXA_JUGADOR(J3_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: BAIXA_JUGADOR NOM3 ', PARCIAL);

      J := PKG_COMPETICIONS.JUGADOR(J3_ID);
      IF (NVL(J.NOM,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: JUGADOR NOM3 NO DISPONIBLE', PARCIAL);

      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_JUGADORS: BAIXA_ZONA Z1 ', PARCIAL);

      RETURN RETORN;
    END;
  END TEST_JUGADORS;


-- ********************
-- TEST_PLANTILLES
--
  FUNCTION TEST_PLANTILLES RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;
      RETURN RETORN;
    END;
  END TEST_PLANTILLES;


-- ********************
-- TEST_PARTITS
--
  FUNCTION TEST_PARTITS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      P PARTITS%ROWTYPE;
      P1_ID NUMBER;
      P2_ID NUMBER;
      P3_ID NUMBER;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;

      -- AFEGIM UNA ZONA
      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z1', 'PRIMERA ZONA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: ALTA_ZONA Z1', PARCIAL);

      -- AFEGIM UNA COMPETICIO
      UNITARI := PKG_COMPETICIONS.ALTA_COMPETICIO('C1', 'PRIMERA COMPETICIO', 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: ALTA_COMPETICIO C1 ', PARCIAL);

      -- AFEGIM UNA TEMPORADA
      UNITARI := PKG_COMPETICIONS.ALTA_TEMPORADA('C1', 2001, TO_DATE('01/01/2001', 'DD/MM/YYYY'), TO_DATE('31/12/2001', 'DD/MM/YYYY'));
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: ALTA_TEMPORADA C1-2001', PARCIAL);

      -- AFEGIM 3 EQUIPS
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ1', 'Equip 1', TO_DATE('01/01/1901', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: ALTA_EQUIP EQ1', PARCIAL);
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ2', 'Equip 2', TO_DATE('02/02/1902', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: ALTA_EQUIP EQ2', PARCIAL);
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ3', 'Equip 3', TO_DATE('03/03/1903', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: ALTA_EQUIP EQ3', PARCIAL);

      -- AFEGIM UN PARTIT
      UNITARI := PKG_PARTITS.ALTA_PARTIT('C1', 2001, 'EQ1', 'EQ2', TO_DATE('12/10/1999', 'DD/MM/YYYY'), '0');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: ALTA_PARTIT EQ1-EQ2', PARCIAL);
      
      -- EL CONSULTEM
      P1_ID := PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2');
      P := PKG_PARTITS.PARTIT(P1_ID);
      PARCIAL := (P.EQUIP1 = 'EQ1') AND (P.EQUIP2 = 'EQ2');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: PARTIT EQ1-EQ2 ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_PARTITS.BAIXA_PARTIT(P1_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: BAIXA_PARTIT EQ1-EQ2 ', PARCIAL);

      P := PKG_PARTITS.PARTIT(P1_ID);
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (P IS NULL) THEN 
      IF (NVL(P.EQUIP1,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: PARTIT EQ1-EQ2 NO DISPONIBLE', PARCIAL);



      -- AFEGIM DOS PARTITS
      UNITARI := PKG_PARTITS.ALTA_PARTIT('C1', 2001, 'EQ2', 'EQ1', TO_DATE('22/03/2014', 'DD/MM/YYYY'), '0');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: ALTA_PARTIT RMA-FCB', PARCIAL);

      UNITARI := PKG_PARTITS.ALTA_PARTIT('C1', 2001, 'EQ3', 'EQ1', TO_DATE('11/01/2014', 'DD/MM/YYYY'), '1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: ALTA_PARTIT ATM-FCB', PARCIAL);
      
      -- ELS CONSULTEM
      P2_ID := PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ2', 'EQ1');
      P := PKG_PARTITS.PARTIT(P2_ID);
      PARCIAL := (P.EQUIP1 = 'EQ2') AND (P.EQUIP2 = 'EQ1');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: PARTIT EQ1-EQ1 ', PARCIAL);

      P3_ID := PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ3', 'EQ1');
      P := PKG_PARTITS.PARTIT(P3_ID);
      PARCIAL := (P.EQUIP1 = 'EQ3') AND (P.EQUIP2 = 'EQ1');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: PARTIT EQ3-EQ1 ', PARCIAL);
      
      -- ELS ESBORREM
      UNITARI := PKG_PARTITS.BAIXA_PARTIT(P2_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: BAIXA_PARTIT EQ2-EQ1 ', PARCIAL);

      P := PKG_PARTITS.PARTIT(P2_ID);
      IF (NVL(P.EQUIP1,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: PARTIT EQ2-EQ1 NO DISPONIBLE', PARCIAL);

      P := PKG_PARTITS.PARTIT(P3_ID);
      PARCIAL := (P.EQUIP1 = 'EQ3') AND (P.EQUIP2 = 'EQ1');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: PARTIT EQ3-EQ1 ', PARCIAL);

      UNITARI := PKG_PARTITS.BAIXA_PARTIT(P3_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: BAIXA_PARTIT EQ3-EQ1 ', PARCIAL);

      P := PKG_PARTITS.PARTIT(P3_ID);
      IF (NVL(P.EQUIP1,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_PARTITS: PARTIT EQ3-EQ1 NO DISPONIBLE', PARCIAL);

      -- ESBORREM ELS EQUIPS
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ3');
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ2');
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ1');
      -- ESBORREM LA TEMPORADA
      UNITARI := PKG_COMPETICIONS.BAIXA_TEMPORADA('C1', 2001);
      -- ESBORREM LA COMPETICIO
      UNITARI := PKG_COMPETICIONS.BAIXA_COMPETICIO('C1');
      -- ESBORREM LA ZONES
      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z1');

      RETURN RETORN;
    END;
  END TEST_PARTITS;


-- ********************
-- TEST_TIPUS_GOL
--
  FUNCTION TEST_TIPUS_GOL RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      TG TIPUS_GOL%ROWTYPE;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;
      
      -- AFEGIM UN TIPUS GOL
      UNITARI := PKG_PARTITS.ALTA_TIPUS_GOL('1', 'PRIMER TIPUS GOL');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: ALTA_TIPUS_GOL 1', PARCIAL);
      
      -- EL CONSULTEM
      TG := PKG_PARTITS.TIP_GOL('1');
      PARCIAL := (TG.DESCRIPCIO = 'PRIMER TIPUS GOL');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: TIPUS_GOL 1 ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_PARTITS.BAIXA_TIPUS_GOL('1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: BAIXA_TIPUS_GOL 1 ', PARCIAL);

      TG := PKG_PARTITS.TIP_GOL('1');
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (TG IS NULL) THEN 
      IF (NVL(TG.DESCRIPCIO,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: TIPUS_GOL 1 NO DISPONIBLE', PARCIAL);



      -- AFEGIM DOS TIPUS GOL
      UNITARI := PKG_PARTITS.ALTA_TIPUS_GOL('2', 'SEGON TIPUS GOL');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: ALTA_TIPUS_GOL 2', PARCIAL);

      UNITARI := PKG_PARTITS.ALTA_TIPUS_GOL('3', 'TERCER TIPUS GOL');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: ALTA_TIPUS_GOL 3', PARCIAL);
      
      -- ELS CONSULTEM
      TG := PKG_PARTITS.TIP_GOL('2');
      PARCIAL := (TG.DESCRIPCIO = 'SEGON TIPUS GOL');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: TIPUS_GOL 2 ', PARCIAL);

      TG := PKG_PARTITS.TIP_GOL('3');
      PARCIAL := (TG.DESCRIPCIO = 'TERCER TIPUS GOL');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: TIPUS_GOL 3 ', PARCIAL);
      
      -- ELS ESBORREM
      UNITARI := PKG_PARTITS.BAIXA_TIPUS_GOL('2');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: BAIXA_TIPUS_GOL 2 ', PARCIAL);

      TG := PKG_PARTITS.TIP_GOL('2');
      IF (NVL(TG.DESCRIPCIO,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: TIPUS_GOL 2 NO DISPONIBLE', PARCIAL);

      TG := PKG_PARTITS.TIP_GOL('3');
      PARCIAL := (TG.DESCRIPCIO = 'TERCER TIPUS GOL');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: TIPUS_GOL 3 ', PARCIAL);

      UNITARI := PKG_PARTITS.BAIXA_TIPUS_GOL('3');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: BAIXA_TIPUS_GOL 3 ', PARCIAL);

      TG := PKG_PARTITS.TIP_GOL('3');
      IF (NVL(TG.DESCRIPCIO,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_TIPUS_GOL: TIPUS_GOL 3 NO DISPONIBLE', PARCIAL);

      RETURN RETORN;
    END;
  END TEST_TIPUS_GOL;


-- ********************
-- TEST_GOLS
--
  FUNCTION TEST_GOLS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      G GOLS%ROWTYPE;
      G1_ID NUMBER;
      G2_ID NUMBER;
      G3_ID NUMBER;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;
      
      -- AFEGIM UNA ZONA
      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z1', 'PRIMERA ZONA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_ZONA Z1', PARCIAL);

      -- AFEGIM UNA COMPETICIO
      UNITARI := PKG_COMPETICIONS.ALTA_COMPETICIO('C1', 'PRIMERA COMPETICIO', 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_COMPETICIO C1 ', PARCIAL);

      -- AFEGIM UNA TEMPORADA
      UNITARI := PKG_COMPETICIONS.ALTA_TEMPORADA('C1', 2001, TO_DATE('01/01/2001', 'DD/MM/YYYY'), TO_DATE('31/12/2001', 'DD/MM/YYYY'));
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_TEMPORADA C1-2001', PARCIAL);

      -- AFEGIM 2 EQUIPS
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ1', 'Equip 1', TO_DATE('01/01/1901', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_EQUIP EQ1 ', PARCIAL);
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ2', 'Equip 2', TO_DATE('02/02/1902', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_EQUIP EQ2 ', PARCIAL);

      -- AFEGIM 2 JUGADORS
      UNITARI := PKG_COMPETICIONS.ALTA_JUGADOR('Nom1', 'Cognom1', '', TO_DATE('11/11/1911', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_JUGADOR Nom1 ', PARCIAL);
      UNITARI := PKG_COMPETICIONS.ALTA_JUGADOR('Nom2', 'Cognom2', '', TO_DATE('22/12/1922', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_JUGADOR Nom2 ', PARCIAL);

      -- AFEGIM UN PARTIT
      UNITARI := PKG_PARTITS.ALTA_PARTIT('C1', 2001, 'EQ1', 'EQ2', TO_DATE('12/10/1999', 'DD/MM/YYYY'), '0');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_PARTIT EQ1-EQ2 ', PARCIAL);
      
      -- AFEGIM 3 TIPUS DE GOL
      UNITARI := PKG_PARTITS.ALTA_TIPUS_GOL('1', 'Primer tipus');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_TIPUS_GOL 1 ', PARCIAL);
      UNITARI := PKG_PARTITS.ALTA_TIPUS_GOL('2', 'Segon tipus');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_TIPUS_GOL 2 ', PARCIAL);
      UNITARI := PKG_PARTITS.ALTA_TIPUS_GOL('3', 'Tercer tipus');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_TIPUS_GOL 3 ', PARCIAL);
     
      -- AFEGIM UN GOL
      UNITARI := PKG_PARTITS.ALTA_GOL( PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'EQ1', PKG_COMPETICIONS.CERCA_JUGADOR('Nom1', 'Cognom1', ''), 1, 35, '1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_GOL Nom1 Partit EQ1-EQ2 ', PARCIAL);

      -- EL CONSULTEM
      G1_ID := PKG_PARTITS.CERCA_GOL( PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 1, 35);
      G := PKG_PARTITS.GOL(G1_ID);
      PARCIAL := (G.TIPUS = '1');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: GOL Nom1 Partit EQ1-EQ2 ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_PARTITS.BAIXA_GOL(G1_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: BAIXA_GOL Nom1 Partit EQ1-EQ2 ', PARCIAL);

      G := PKG_PARTITS.GOL(G1_ID);
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (G IS NULL) THEN 
      IF (NVL(G.TIPUS,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: GOL Nom1 Partit EQ1-EQ2 NO DISPONIBLE ', PARCIAL);



      -- AFEGIM DOS GOLS
      UNITARI := PKG_PARTITS.ALTA_GOL( PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'EQ2', PKG_COMPETICIONS.CERCA_JUGADOR('Nom2', 'Cognom2', ''), 1, 40, '2');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_GOL Nom2 Partit EQ1-EQ2 ', PARCIAL);

      UNITARI := PKG_PARTITS.ALTA_GOL( PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'EQ1', PKG_COMPETICIONS.CERCA_JUGADOR('Nom1', 'Cognom1', ''), 2, 67, '3');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: ALTA_GOL Nom1 Partit EQ1-EQ2 ', PARCIAL);
      
      -- ELS CONSULTEM
      G2_ID := PKG_PARTITS.CERCA_GOL( PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 1, 40);
      G := PKG_PARTITS.GOL(G2_ID);
      PARCIAL := (G.TIPUS = '2');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: GOL Nom2 Partit EQ1-EQ2 ', PARCIAL);

      G3_ID := PKG_PARTITS.CERCA_GOL( PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 2, 67);
      G := PKG_PARTITS.GOL(G3_ID);
      PARCIAL := (G.TIPUS = '3');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: GOL Nom1 Partit EQ1-EQ2 ', PARCIAL);
      
      -- ELS ESBORREM
      UNITARI := PKG_PARTITS.BAIXA_GOL(G2_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: BAIXA_GOL Nom2 Partit EQ1-EQ2 ', PARCIAL);

      G := PKG_PARTITS.GOL(G2_ID);
      IF (NVL(G.TIPUS,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: GOL Nom2 Partit EQ1-EQ2 NO DISPONIBLE', PARCIAL);

      G := PKG_PARTITS.GOL(G3_ID);
      PARCIAL := (G.TIPUS = '3');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: GOL Nom1 Partit EQ1-EQ2 ', PARCIAL);

      UNITARI := PKG_PARTITS.BAIXA_GOL(G3_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: BAIXA_GOL Nom1 Partit EQ1-EQ2 ', PARCIAL);

      G := PKG_PARTITS.GOL(G3_ID);
      IF (NVL(G.TIPUS,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_GOLS: GOL Nom1 Partit EQ1-EQ2 NO DISPONIBLE', PARCIAL);

      -- ESBORREM ELS TIPUS DE GOL
      UNITARI := PKG_PARTITS.BAIXA_TIPUS_GOL('1');
      UNITARI := PKG_PARTITS.BAIXA_TIPUS_GOL('2');
      UNITARI := PKG_PARTITS.BAIXA_TIPUS_GOL('3');
      -- ESBORREM EL PARTIT
      UNITARI := PKG_PARTITS.BAIXA_PARTIT( PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2') );
      -- ESBORREM ELS JUGADORS
      UNITARI := PKG_COMPETICIONS.BAIXA_JUGADOR( PKG_COMPETICIONS.CERCA_JUGADOR('Nom1', 'Cognom1', '') );
      UNITARI := PKG_COMPETICIONS.BAIXA_JUGADOR( PKG_COMPETICIONS.CERCA_JUGADOR('Nom2', 'Cognom2', '') );
      -- ESBORREM ELS EQUIPS
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ1');
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ2');
      -- ESBORREM LA TEMPORADA
      UNITARI := PKG_COMPETICIONS.BAIXA_TEMPORADA('C1', 2001);
      -- ESBORREM LA COMPETICIO
      UNITARI := PKG_COMPETICIONS.BAIXA_COMPETICIO('C1');
      -- ESBORREM LA ZONA
      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z1');

      RETURN RETORN;
    END;
  END TEST_GOLS;


-- ********************
-- TEST_USUARIS
--
  FUNCTION TEST_USUARIS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      U USUARIS%ROWTYPE;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;
      
      -- AFEGIM UNA ZONA
      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z1', 'PRIMERA ZONA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: ALTA_ZONA Z1 ', PARCIAL);
      
      -- AFEGIM UN USUARI
      UNITARI := PKG_USUARIS.ALTA_USUARI('USERNAME@UOC.COM', 'CONTRASENYA', 'NOM', 'COGNOM1', 'COGNOM2', 'H', 'D', '12345678Z', TO_DATE('01/01/1970', 'DD/MM/YYYY'), 'Z1', 'Z1', 'IBAN', 'TARGETA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: ALTA_USUARI USERNAME@UOC.COM ', PARCIAL);
      
      -- EL CONSULTEM
      U := PKG_USUARIS.USUARI('USERNAME@UOC.COM');
      PARCIAL := (U.NOM_USUARI = 'USERNAME@UOC.COM');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: USUARI USERNAME@UOC.COM ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_USUARIS.BAIXA_USUARI('USERNAME@UOC.COM');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: BAIXA_USUARI USERNAME@UOC.COM ', PARCIAL);

      U := PKG_USUARIS.USUARI('USERNAME@UOC.COM');
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (U IS NULL) THEN 
      IF (NVL(U.NOM_USUARI,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: USUARI USERNAME@UOC.COM NO DISPONIBLE', PARCIAL);



      -- AFEGIM DOS COMPETICIONS
      UNITARI := PKG_USUARIS.ALTA_USUARI('U2@UOC.COM', 'CONTRASENYA', 'NOM_U2', 'COGNOM1', 'COGNOM2', 'H', 'D', '22222222B', TO_DATE('01/01/1970', 'DD/MM/YYYY'), 'Z1', 'Z1', 'IBAN', 'TARGETA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: ALTA_USUARI U2@UOC.COM', PARCIAL);

      UNITARI := PKG_USUARIS.ALTA_USUARI('U3@UOC.COM', 'CONTRASENYA', 'NOM_U3', 'COGNOM1', 'COGNOM2', 'H', 'D', '33333333C', TO_DATE('01/01/1970', 'DD/MM/YYYY'), 'Z1', 'Z1', 'IBAN', 'TARGETA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: ALTA_USUARI U3@UOC.COM', PARCIAL);
      
      -- ELS CONSULTEM
      U := PKG_USUARIS.USUARI('U2@UOC.COM');
      PARCIAL := (U.NOM_USUARI = 'U2@UOC.COM');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: USUARI U2@UOC.COM ', PARCIAL);

      U := PKG_USUARIS.USUARI('U3@UOC.COM');
      PARCIAL := (U.NOM_USUARI = 'U3@UOC.COM');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: USUARI U3@UOC.COM ', PARCIAL);
      
      -- ELS ESBORREM
      UNITARI := PKG_USUARIS.BAIXA_USUARI('U2@UOC.COM');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: BAIXA_USUARI U2@UOC.COM ', PARCIAL);

      U := PKG_USUARIS.USUARI('U2@UOC.COM');
      IF (NVL(U.NOM_USUARI,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: USUARI U2@UOC.COM NO DISPONIBLE', PARCIAL);

      U := PKG_USUARIS.USUARI('U3@UOC.COM');
      PARCIAL := (U.NOM_USUARI = 'U3@UOC.COM');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: USUARI U3@UOC.COM ', PARCIAL);

      UNITARI := PKG_USUARIS.BAIXA_USUARI('U3@UOC.COM');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: BAIXA_USUARI U3@UOC.COM ', PARCIAL);

      U := PKG_USUARIS.USUARI('U3@UOC.COM');
      IF (NVL(U.NOM_USUARI,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: USUARI U3@UOC.COM NO DISPONIBLE', PARCIAL);

      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_USUARIS: BAIXA_ZONA Z1 ', PARCIAL);

      RETURN RETORN;
    END;
  END TEST_USUARIS;


-- ********************
-- TEST_MODALITATS
--
  FUNCTION TEST_MODALITATS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      M MODALITATS%ROWTYPE;
      M1_ID NUMBER;
      M2_ID NUMBER;
      M3_ID NUMBER;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;
      M1_ID := 0;
      M2_ID := 0;
      M3_ID := 0;
      
      -- AFEGIM UNA MODALITAT
      UNITARI := PKG_APOSTES.ALTA_MODALITAT('PRIMERA MODALITAT');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: ALTA_MODALITAT PRIMERA MODALITAT', PARCIAL);
      
      -- LA CONSULTEM
      M1_ID := PKG_APOSTES.CERCA_MODALITAT('PRIMERA MODALITAT');
      M := PKG_APOSTES.MODALITAT(M1_ID);
      PARCIAL := (M.DESCRIPCIO = 'PRIMERA MODALITAT');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: MODALITAT PRIMERA MODALITAT ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_APOSTES.BAIXA_MODALITAT(M1_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: BAIXA_MODALITAT PRIMERA MODALITAT ', PARCIAL);

      M := PKG_APOSTES.MODALITAT(M1_ID);
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (M IS NULL) THEN 
      IF (NVL(M.DESCRIPCIO,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: ZONA PRIMERA MODALITAT NO DISPONIBLE', PARCIAL);



      -- AFEGIM DUES MODALITATS
      UNITARI := PKG_APOSTES.ALTA_MODALITAT('SEGONA MODALITAT');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: ALTA_MODALITAT SEGONA MODALITAT', PARCIAL);

      UNITARI := PKG_APOSTES.ALTA_MODALITAT('TERCERA MODALITAT');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: ALTA_MODALITAT TERCERA MODALITAT', PARCIAL);
      
      -- LES CONSULTEM
      M2_ID := PKG_APOSTES.CERCA_MODALITAT('SEGONA MODALITAT');
      M := PKG_APOSTES.MODALITAT(M2_ID);
      PARCIAL := (M.DESCRIPCIO = 'SEGONA MODALITAT');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: MODALITAT SEGONA MODALITAT ', PARCIAL);

      M3_ID := PKG_APOSTES.CERCA_MODALITAT('TERCERA MODALITAT');
      M := PKG_APOSTES.MODALITAT(M3_ID);
      PARCIAL := (M.DESCRIPCIO = 'TERCERA MODALITAT');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: MODALITAT TERCERA MODALITAT ', PARCIAL);
      
      -- LES ESBORREM
      UNITARI := PKG_APOSTES.BAIXA_MODALITAT(M2_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: BAIXA_MODALITAT SEGONA MODALITAT ', PARCIAL);

      M := PKG_APOSTES.MODALITAT(M2_ID);
      IF (NVL(M.DESCRIPCIO,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: MODALITAT SEGONA MODALITAT NO DISPONIBLE', PARCIAL);

      M := PKG_APOSTES.MODALITAT(M3_ID);
      PARCIAL := (M.DESCRIPCIO = 'TERCERA MODALITAT');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: MODALITAT TERCERA MODALITAT ', PARCIAL);

      UNITARI := PKG_APOSTES.BAIXA_MODALITAT(M3_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: BAIXA_MODALITAT TERCERA MODALITAT ', PARCIAL);

      M := PKG_APOSTES.MODALITAT(M3_ID);
      IF (NVL(M.DESCRIPCIO,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_MODALITATS: MODALITAT TERCERA MODALITAT NO DISPONIBLE', PARCIAL);

      RETURN RETORN;
    END;
  END TEST_MODALITATS;


-- ********************
-- TEST_COEFICIENTS
--
  FUNCTION TEST_COEFICIENTS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      C COEFICIENTS%ROWTYPE;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;

      --RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1', 1.75 );
      --RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), 'X', 1.95 );
      --RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '2', 2.15 );
      
      -- AFEGIM UNA ZONA
      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z1', 'PRIMERA ZONA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: ALTA_ZONA Z1', PARCIAL);

      -- AFEGIM UNA COMPETICIO
      UNITARI := PKG_COMPETICIONS.ALTA_COMPETICIO('C1', 'PRIMERA COMPETICIO', 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: ALTA_COMPETICIO C1 ', PARCIAL);

      -- AFEGIM UNA TEMPORADA
      UNITARI := PKG_COMPETICIONS.ALTA_TEMPORADA('C1', 2001, TO_DATE('01/01/2001', 'DD/MM/YYYY'), TO_DATE('31/12/2001', 'DD/MM/YYYY'));
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: ALTA_TEMPORADA C1-2001', PARCIAL);

      -- AFEGIM 2 EQUIPS
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ1', 'Equip 1', TO_DATE('01/01/1901', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: ALTA_EQUIP EQ1', PARCIAL);
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ2', 'Equip 2', TO_DATE('02/02/1902', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: ALTA_EQUIP EQ2', PARCIAL);

      -- AFEGIM UN PARTIT
      UNITARI := PKG_PARTITS.ALTA_PARTIT('C1', 2001, 'EQ1', 'EQ2', TO_DATE('12/10/1999', 'DD/MM/YYYY'), '0');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: ALTA_PARTIT EQ1-EQ2', PARCIAL);
          
      -- AFEGIM UNA MODALITAT
      UNITARI := PKG_APOSTES.ALTA_MODALITAT('MODALITAT1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: ALTA_MODALITAT MODALITAT1', PARCIAL);

      -- AFEGIM UN COEFICIENT
      UNITARI := PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R1', 1.75);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: ALTA_COEFICIENT Modalitat1 R1 Partit EQ1-EQ2 ', PARCIAL);

      -- EL CONSULTEM
      C := PKG_APOSTES.COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R1');
      PARCIAL := (C.VALOR = 1.75);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: COEFICIENT Modalitat1 R1 Partit EQ1-EQ2 ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_APOSTES.BAIXA_COEFICIENT(PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: BAIXA_COEFICIENT Modalitat1 R1 Partit EQ1-EQ2 ', PARCIAL);

      C := PKG_APOSTES.COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R1');
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (C IS NULL) THEN 
      IF (NVL(C.RESULTAT,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: COEFICIENT Modalitat1 R1 Partit EQ1-EQ2 NO DISPONIBLE ', PARCIAL);



      -- AFEGIM DOS GOLS
      UNITARI := PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R2', 1.95);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: ALTA_COEFICIENT Modalitat1 R2 Partit EQ1-EQ2 ', PARCIAL);

      UNITARI := PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R3', 2.15);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: ALTA_COEFICIENT Modalitat1 R3 Partit EQ1-EQ2 ', PARCIAL);
      
      -- ELS CONSULTEM
      C := PKG_APOSTES.COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R2');
      PARCIAL := (C.VALOR = 1.95);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: COEFICIENT Modalitat1 R2 Partit EQ1-EQ2 ', PARCIAL);

      C := PKG_APOSTES.COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R3');
      PARCIAL := (C.VALOR = 2.15);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: COEFICIENT Modalitat1 R3 Partit EQ1-EQ2 ', PARCIAL);
      
      -- ELS ESBORREM
      UNITARI := PKG_APOSTES.BAIXA_COEFICIENT(PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R2');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: BAIXA_COEFICIENT Modalitat1 R2 Partit EQ1-EQ2 ', PARCIAL);

      C := PKG_APOSTES.COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R2');
      IF (NVL(C.VALOR,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: COEFICIENT Modalitat1 R2 Partit EQ1-EQ2 NO DISPONIBLE', PARCIAL);

      C := PKG_APOSTES.COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R3');
      PARCIAL := (C.VALOR = 2.15);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: COEFICIENT Modalitat1 R3 Partit EQ1-EQ2 ', PARCIAL);

      UNITARI := PKG_APOSTES.BAIXA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R3');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: BAIXA_COEFICIENT Modalitat1 R3 Partit EQ1-EQ2 ', PARCIAL);

      C := PKG_APOSTES.COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R3');
      IF (NVL(C.VALOR,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_COEFICIENTS: COEFICIENT Modalitat1 R3 Partit EQ1-EQ2 NO DISPONIBLE', PARCIAL);

      -- ESBORREM LA MODALITAT
      UNITARI := PKG_APOSTES.BAIXA_MODALITAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1') );
      -- ESBORREM EL PARTIT
      UNITARI := PKG_PARTITS.BAIXA_PARTIT( PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2') );
      -- ESBORREM ELS EQUIPS
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ1');
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ2');
      -- ESBORREM LA TEMPORADA
      UNITARI := PKG_COMPETICIONS.BAIXA_TEMPORADA('C1', 2001);
      -- ESBORREM LA COMPETICIO
      UNITARI := PKG_COMPETICIONS.BAIXA_COMPETICIO('C1');
      -- ESBORREM LA ZONA
      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z1');

      RETURN RETORN;
    END;
  END TEST_COEFICIENTS;


-- ********************
-- TEST_APOSTES
--
  FUNCTION TEST_APOSTES RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      A APOSTES%ROWTYPE;
      A1_ID NUMBER;
      A2_ID NUMBER;
      A3_ID NUMBER;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;

      --RETORN := RETORN + PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1', 10.00 );
      --RETORN := RETORN + PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '2-0', 5.00 );
      
      -- AFEGIM UNA ZONA
      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z1', 'PRIMERA ZONA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_ZONA Z1', PARCIAL);

      -- AFEGIM UNA COMPETICIO
      UNITARI := PKG_COMPETICIONS.ALTA_COMPETICIO('C1', 'PRIMERA COMPETICIO', 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_COMPETICIO C1 ', PARCIAL);

      -- AFEGIM UNA TEMPORADA
      UNITARI := PKG_COMPETICIONS.ALTA_TEMPORADA('C1', 2001, TO_DATE('01/01/2001', 'DD/MM/YYYY'), TO_DATE('31/12/2001', 'DD/MM/YYYY'));
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_TEMPORADA C1-2001', PARCIAL);

      -- AFEGIM 2 EQUIPS
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ1', 'Equip 1', TO_DATE('01/01/1901', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_EQUIP EQ1', PARCIAL);
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ2', 'Equip 2', TO_DATE('02/02/1902', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_EQUIP EQ2', PARCIAL);

      -- AFEGIM UN PARTIT
      UNITARI := PKG_PARTITS.ALTA_PARTIT('C1', 2001, 'EQ1', 'EQ2', TO_DATE('12/10/1999', 'DD/MM/YYYY'), '0');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_PARTIT EQ1-EQ2', PARCIAL);
          
      -- AFEGIM TRES MODALITAT
      UNITARI := PKG_APOSTES.ALTA_MODALITAT('MODALITAT1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_MODALITAT MODALITAT1', PARCIAL);
      UNITARI := PKG_APOSTES.ALTA_MODALITAT('MODALITAT2');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_MODALITAT MODALITAT2', PARCIAL);
      UNITARI := PKG_APOSTES.ALTA_MODALITAT('MODALITAT3');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_MODALITAT MODALITAT3', PARCIAL);

      -- AFEGIM UN USUARI
      UNITARI := PKG_USUARIS.ALTA_USUARI('USERNAME@UOC.COM', 'CONTRASENYA', 'NOM', 'COGNOM1', 'COGNOM2', 'H', 'D', '12345678Z', TO_DATE('01/01/1970', 'DD/MM/YYYY'), 'Z1', 'Z1', 'IBAN', 'TARGETA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_USUARI USERNAME@UOC.COM', PARCIAL);
      UNITARI := PKG_JOC.LOGIN('USERNAME@UOC.COM', 'CONTRASENYA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_USUARI LOGIN USERNAME@UOC.COM', PARCIAL);
      

      -- AFEGIM UNA APOSTA
      UNITARI := PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'A1', 1.75);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_APOSTA Modalitat1 A1 Partit EQ1-EQ2 ', PARCIAL);

      -- EL CONSULTEM
      A1_ID := PKG_APOSTES.CERCA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
      A := PKG_APOSTES.APOSTA(A1_ID);
      PARCIAL := (A.IMPORT = 1.75);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: APOSTA Modalitat1 A1 Partit EQ1-EQ2 ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_APOSTES.BAIXA_APOSTA(A1_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: BAIXA_APOSTA Modalitat1 A1 Partit EQ1-EQ2 ', PARCIAL);

      A := PKG_APOSTES.APOSTA(A1_ID);
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (A IS NULL) THEN 
      IF (NVL(A.IMPORT,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: APOSTA Modalitat1 A1 Partit EQ1-EQ2 NO DISPONIBLE ', PARCIAL);



      -- AFEGIM DOS APOSTES
      UNITARI := PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('MODALITAT2'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'A2', 1.95);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_APOSTA Modalitat1 A2 Partit EQ1-EQ2 ', PARCIAL);

      UNITARI := PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('MODALITAT3'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'A3', 2.15);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: ALTA_APOSTA Modalitat1 A3 Partit EQ1-EQ2 ', PARCIAL);
      
      -- ELS CONSULTEM
      A2_ID := PKG_APOSTES.CERCA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('MODALITAT2'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
      A := PKG_APOSTES.APOSTA(A2_ID);
      PARCIAL := (A.IMPORT = 1.95);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: APOSTA Modalitat1 A2 Partit EQ1-EQ2 ', PARCIAL);

      A3_ID := PKG_APOSTES.CERCA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('MODALITAT3'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
      A := PKG_APOSTES.APOSTA(A3_ID);
      PARCIAL := (A.IMPORT = 2.15);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: APOSTA Modalitat1 A3 Partit EQ1-EQ2 ', PARCIAL);
      
      -- ELS ESBORREM
      UNITARI := PKG_APOSTES.BAIXA_APOSTA(A2_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: BAIXA_APOSTA Modalitat1 A2 Partit EQ1-EQ2 ', PARCIAL);

      A := PKG_APOSTES.APOSTA(A2_ID);
      IF (NVL(A.IMPORT,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: APOSTA Modalitat1 A2 Partit EQ1-EQ2 NO DISPONIBLE', PARCIAL);

      A := PKG_APOSTES.APOSTA(A3_ID);
      PARCIAL := (A.IMPORT = 2.15);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: APOSTA Modalitat1 A3 Partit EQ1-EQ2 ', PARCIAL);

      UNITARI := PKG_APOSTES.BAIXA_APOSTA(A3_ID);
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: BAIXA_APOSTA Modalitat1 A3 Partit EQ1-EQ2 ', PARCIAL);

      A := PKG_APOSTES.APOSTA(A3_ID);
      IF (NVL(A.IMPORT,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_APOSTES: APOSTA Modalitat1 A3 Partit EQ1-EQ2 NO DISPONIBLE', PARCIAL);

      -- ESBORREM L'USUARI
      UNITARI := PKG_USUARIS.BAIXA_USUARI('USERNAME@UOC.COM');
      -- ESBORREM LES MODALITATS
      UNITARI := PKG_APOSTES.BAIXA_MODALITAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1') );
      UNITARI := PKG_APOSTES.BAIXA_MODALITAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT2') );
      UNITARI := PKG_APOSTES.BAIXA_MODALITAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT3') );
      -- ESBORREM EL PARTIT
      UNITARI := PKG_PARTITS.BAIXA_PARTIT( PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2') );
      -- ESBORREM ELS EQUIPS
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ1');
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ2');
      -- ESBORREM LA TEMPORADA
      UNITARI := PKG_COMPETICIONS.BAIXA_TEMPORADA('C1', 2001);
      -- ESBORREM LA COMPETICIO
      UNITARI := PKG_COMPETICIONS.BAIXA_COMPETICIO('C1');
      -- ESBORREM LA ZONA
      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z1');

      UNITARI := PKG_JOC.LOGOUT;

      RETURN RETORN;
    END;
  END TEST_APOSTES;


-- ********************
-- TEST_RESULTATS
--
  FUNCTION TEST_RESULTATS RETURN BOOLEAN IS
  BEGIN
    DECLARE
      PARCIAL BOOLEAN;
      RETORN BOOLEAN;
      UNITARI NUMBER;
      R RESULTATS%ROWTYPE;
    BEGIN
      PARCIAL := TRUE;
      RETORN := TRUE;
      UNITARI := 0;
      
      --RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1');

      -- AFEGIM UNA ZONA
      UNITARI := PKG_COMPETICIONS.ALTA_ZONA('Z1', 'PRIMERA ZONA');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: ALTA_ZONA Z1', PARCIAL);

      -- AFEGIM UNA COMPETICIO
      UNITARI := PKG_COMPETICIONS.ALTA_COMPETICIO('C1', 'PRIMERA COMPETICIO', 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: ALTA_COMPETICIO C1 ', PARCIAL);

      -- AFEGIM UNA TEMPORADA
      UNITARI := PKG_COMPETICIONS.ALTA_TEMPORADA('C1', 2001, TO_DATE('01/01/2001', 'DD/MM/YYYY'), TO_DATE('31/12/2001', 'DD/MM/YYYY'));
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: ALTA_TEMPORADA C1-2001', PARCIAL);

      -- AFEGIM 2 EQUIPS
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ1', 'Equip 1', TO_DATE('01/01/1901', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: ALTA_EQUIP EQ1', PARCIAL);
      UNITARI := PKG_COMPETICIONS.ALTA_EQUIP('EQ2', 'Equip 2', TO_DATE('02/02/1902', 'DD/MM/YYYY'), 'Z1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: ALTA_EQUIP EQ2', PARCIAL);

      -- AFEGIM UN PARTIT
      UNITARI := PKG_PARTITS.ALTA_PARTIT('C1', 2001, 'EQ1', 'EQ2', TO_DATE('12/10/1999', 'DD/MM/YYYY'), '0');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: ALTA_PARTIT EQ1-EQ2', PARCIAL);
          
      -- AFEGIM TRES MODALITATS
      UNITARI := PKG_APOSTES.ALTA_MODALITAT('MODALITAT1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: ALTA_MODALITAT MODALITAT1', PARCIAL);
      UNITARI := PKG_APOSTES.ALTA_MODALITAT('MODALITAT2');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: ALTA_MODALITAT MODALITAT2', PARCIAL);
      UNITARI := PKG_APOSTES.ALTA_MODALITAT('MODALITAT3');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: ALTA_MODALITAT MODALITAT3', PARCIAL);

      -- AFEGIM UN RESULTAT
      UNITARI := PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R1');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: ALTA_RESULTAT Modalitat1 R1 Partit EQ1-EQ2 ', PARCIAL);

      -- EL CONSULTEM
      R := PKG_APOSTES.RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
      PARCIAL := (R.RESULTAT = 'R1');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: RESULTAT Modalitat1 R1 Partit EQ1-EQ2 ', PARCIAL);
      
      -- L'ESBORREM
      UNITARI := PKG_APOSTES.BAIXA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: BAIXA_RESULTAT Modalitat1 R1 Partit EQ1-EQ2 ', PARCIAL);

      R := PKG_APOSTES.RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
-- LA COMPARACIO AMB NULL NO ES TRIVIAL:
--      IF (C IS NULL) THEN 
      IF (NVL(R.RESULTAT,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: RESULTAT Modalitat1 R1 Partit EQ1-EQ2 NO DISPONIBLE ', PARCIAL);



      -- AFEGIM DOS RESULTATS
      UNITARI := PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT2'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R2');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: ALTA_RESULTAT Modalitat2 R2 Partit EQ1-EQ2 ', PARCIAL);

      UNITARI := PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT3'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'), 'R3');
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: ALTA_RESULTAT Modalitat3 R3 Partit EQ1-EQ2 ', PARCIAL);
      
      -- ELS CONSULTEM
      R := PKG_APOSTES.RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT2'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
      PARCIAL := (R.RESULTAT = 'R2');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: RESULTAT Modalitat2 R2 Partit EQ1-EQ2 ', PARCIAL);

      R := PKG_APOSTES.RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT3'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
      PARCIAL := (R.RESULTAT = 'R3');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: RESULTAT Modalitat3 R3 Partit EQ1-EQ2 ', PARCIAL);
      
      -- ELS ESBORREM
      UNITARI := PKG_APOSTES.BAIXA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT2'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: BAIXA_RESULTAT Modalitat2 R2 Partit EQ1-EQ2 ', PARCIAL);

      R := PKG_APOSTES.RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT2'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
      IF (NVL(R.RESULTAT,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: RESULTAT Modalitat2 R2 Partit EQ1-EQ2 NO DISPONIBLE', PARCIAL);

      R := PKG_APOSTES.RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT3'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
      PARCIAL := (R.RESULTAT = 'R3');
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: RESULTAT Modalitat3 R3 Partit EQ1-EQ2 ', PARCIAL);

      UNITARI := PKG_APOSTES.BAIXA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT3'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
      PARCIAL := (UNITARI = 0);
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: BAIXA_RESULTAT Modalitat3 R3 Partit EQ1-EQ2 ', PARCIAL);

      R := PKG_APOSTES.RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT3'), PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2'));
      IF (NVL(R.RESULTAT,1)=1) THEN 
        PARCIAL := TRUE;
      ELSE
        PARCIAL := FALSE;
      END IF;
      RETORN := RETORN AND PARCIAL;
      DEBUG('TEST_RESULTATS: RESULTAT Modalitat3 R3 Partit EQ1-EQ2 NO DISPONIBLE', PARCIAL);

      -- ESBORREM LES MODALITATS
      UNITARI := PKG_APOSTES.BAIXA_MODALITAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT1') );
      UNITARI := PKG_APOSTES.BAIXA_MODALITAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT2') );
      UNITARI := PKG_APOSTES.BAIXA_MODALITAT( PKG_APOSTES.CERCA_MODALITAT('MODALITAT3') );
      -- ESBORREM EL PARTIT
      UNITARI := PKG_PARTITS.BAIXA_PARTIT( PKG_PARTITS.CERCA_PARTIT('C1', 2001, 'EQ1', 'EQ2') );
      -- ESBORREM ELS EQUIPS
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ1');
      UNITARI := PKG_COMPETICIONS.BAIXA_EQUIP('EQ2');
      -- ESBORREM LA TEMPORADA
      UNITARI := PKG_COMPETICIONS.BAIXA_TEMPORADA('C1', 2001);
      -- ESBORREM LA COMPETICIO
      UNITARI := PKG_COMPETICIONS.BAIXA_COMPETICIO('C1');
      -- ESBORREM LA ZONA
      UNITARI := PKG_COMPETICIONS.BAIXA_ZONA('Z1');

      RETURN RETORN;
    END;
  END TEST_RESULTATS;


-- ********************
-- DADES INICIALS
-- INSERCIO DE LES DADES INICIALS DEL SISTEMA PER PODER FER PROVES
--
  FUNCTION DADES_INICIALS_ZONES RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('CAT', 'Catalunya');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('ES' , 'Espaa');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('PT' , 'Portugal');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('EU' , 'Europa');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('AR' , 'Argentina');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('SAM', 'Sudamrica');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('NAM', 'Nordamrica');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('AFR', 'frica');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('ASI', 'sia');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('OCE', 'Oceania');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('AQ' , 'Antrtida');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('MON', 'Mundial');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_ZONA('OLI', 'Olimpiades');
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_ZONES');
      END IF;
      
      RETURN RETORN;
    END;
  END DADES_INICIALS_ZONES;

  FUNCTION DADES_INICIALS_COMPETICIONS RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_COMPETICIO('BBVA', 'Lliga Espaola de primera divisi','ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_COMPETICIO('ADELANTE', 'Lliga Espaola de segona divisi','ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_COMPETICIO('COPA', 'Copa d''Espaa','ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_COMPETICIO('COPA FINAL', 'Copa d''Espaa fase final','ES');
      
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_COMPETICIO('CHAMPIONS LEAGUE', 'Champions League','EU');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_COMPETICIO('CHAMPIONS FINAL', 'Champions League fase final','EU');
      
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_COMPETICIO('LIBERTADORES LEAGUE', 'Copa Libertadores','SAM');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_COMPETICIO('LIBERTADORES FINAL', 'Copa Libertadores fase final','SAM');

      RETORN := RETORN + PKG_COMPETICIONS.ALTA_COMPETICIO('MUNDIAL', 'Mundial','MON');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_COMPETICIO('MUNDIAL FINAL', 'Mundial fase final','MON');
      
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_COMPETICIO('OLIMPIADA', 'Olimpiades','OLI');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_COMPETICIO('OLIMPIADA FINAL', 'Olimpiades fase final','OLI');
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_COMPETICIONS');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_COMPETICIONS;

  FUNCTION DADES_INICIALS_TEMPORADES RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_TEMPORADA('BBVA', 2013, TO_DATE('08/09/2013', 'DD/MM/YYYY'), TO_DATE('19/06/2014', 'DD/MM/YYYY'));
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_TEMPORADA('CHAMPIONS LEAGUE', 2013, TO_DATE('07/10/2013', 'DD/MM/YYYY'), TO_DATE('21/05/2014', 'DD/MM/YYYY'));
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_COMPETICIONS');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_TEMPORADES;

  FUNCTION DADES_INICIALS_EQUIPS RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('FCB', 'Futbol Club Barcelona', TO_DATE('03/05/1899', 'DD/MM/YYYY'), 'CAT');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('RMA', 'Real Madrid', TO_DATE('04/05/1900', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('SEV', 'Sevilla Ftbol Club', TO_DATE('06/07/1890', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('ATH', 'Athletic Club de Bilbao', TO_DATE('08/09/1898', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('ESP', 'Real Club Esportiu Espanyol', TO_DATE('10/11/1900', 'DD/MM/YYYY'), 'CAT');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('ATM', 'Club Atltico de Madrid', TO_DATE('12/01/1903', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('BET', 'Real Betic Balompi', TO_DATE('14/02/1907', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('RSO', 'Real Sociedad de Ftbol', TO_DATE('16/04/1909', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('LEV', 'Levante Unin Deportiva', TO_DATE('18/06/1909', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('VAL', 'Valencia Club de Ftbol', TO_DATE('20/08/1919', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('OSA', 'Club Atltico Osasuna', TO_DATE('21/09/1920', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('ELC', 'Elche Club de Ftbol', TO_DATE('24/10/1922', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('CEL', 'Real Club Celta de Vigo', TO_DATE('26/12/1923', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('RAY', 'Rayo Vallecano de Madrid', TO_DATE('28/02/1924', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('VLL', 'Real Valladolid Club de Ftbol', TO_DATE('31/05/1928', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('GRA', 'Granada Club de Ftbol', TO_DATE('03/06/1931', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('VIL', 'Villareal Club de Ftbol', TO_DATE('07/09/1944', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('GET', 'Getafe Club de Ftbol', TO_DATE('11/11/1983', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('ALM', 'Unin Deportiva Almeria', TO_DATE('13/01/1989', 'DD/MM/YYYY'), 'ES');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_EQUIP('MAL', 'Mlaga Club de Ftbol', TO_DATE('15/02/1994', 'DD/MM/YYYY'), 'ES');
--deportivo
--mallorca
--zaragoza
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_EQUIPS');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_EQUIPS;

  FUNCTION DADES_INICIALS_JUGADORS RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_JUGADOR('Lionel', 'Messi', '', TO_DATE('12/10/1993', 'DD/MM/YYYY'), 'AR');
      RETORN := RETORN + PKG_COMPETICIONS.ALTA_JUGADOR('Cristiano', 'Ronaldo', '', TO_DATE('24/03/1991', 'DD/MM/YYYY'), 'PT');
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_JUGADORS');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_JUGADORS;

  FUNCTION DADES_INICIALS_PLANTILLES RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_PLANTILLES');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_PLANTILLES;

  FUNCTION DADES_INICIALS_PARTITS RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
-- JORNADA 1 Sb. 17 y Dom. 18 Agosto
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','LEV', TO_DATE('17/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','MAL', TO_DATE('17/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','BET', TO_DATE('17/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','ATM', TO_DATE('17/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','GET', TO_DATE('17/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','ATH', TO_DATE('18/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','GRA', TO_DATE('18/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','ESP', TO_DATE('18/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','VIL', TO_DATE('18/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','ELC', TO_DATE('18/08/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 2 Sb. 24 y Dom. 25 Agosto
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','VAL', TO_DATE('24/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','CEL', TO_DATE('24/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','RMA', TO_DATE('24/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','OSA', TO_DATE('24/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','VLL', TO_DATE('24/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','ALM', TO_DATE('25/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','RSO', TO_DATE('25/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','RAY', TO_DATE('25/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','SEV', TO_DATE('25/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','FCB', TO_DATE('25/08/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 3 Sb. 31 y Dom. 1 Septiembre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','ATH', TO_DATE('31/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','GET', TO_DATE('31/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','ELC', TO_DATE('31/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','ATM', TO_DATE('31/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','LEV', TO_DATE('31/08/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','MAL', TO_DATE('01/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','FCB', TO_DATE('01/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','BET', TO_DATE('01/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','GRA', TO_DATE('01/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','VIL', TO_DATE('01/09/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 4 Sb. 14 y Dom. 15 Septiembre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','VAL', TO_DATE('14/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','ESP', TO_DATE('14/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','CEL', TO_DATE('14/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','RMA', TO_DATE('14/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','OSA', TO_DATE('14/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','VLL', TO_DATE('15/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','ALM', TO_DATE('15/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','RSO', TO_DATE('15/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','SEV', TO_DATE('15/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','RAY', TO_DATE('15/09/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 5 Sb. 21 y Dom. 22 Septiembre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','ATH', TO_DATE('21/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','VIL', TO_DATE('21/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','GET', TO_DATE('21/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','ELC', TO_DATE('21/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','GRA', TO_DATE('21/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','ATM', TO_DATE('22/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','LEV', TO_DATE('22/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','MAL', TO_DATE('22/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','FCB', TO_DATE('22/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','SEV', TO_DATE('22/09/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 6 Mar. 24 y Mier. 25 Septiembre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','CEL', TO_DATE('24/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','ALM', TO_DATE('24/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','RSO', TO_DATE('24/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','OSA', TO_DATE('24/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','VAL', TO_DATE('24/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','BET', TO_DATE('25/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','ESP', TO_DATE('25/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','RMA', TO_DATE('25/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','VLL', TO_DATE('25/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','RAY', TO_DATE('25/09/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 7 Sb. 28 y Dom. 29 Septiembre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','ELC', TO_DATE('28/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','ATH', TO_DATE('28/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','FCB', TO_DATE('28/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','VIL', TO_DATE('28/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','GET', TO_DATE('28/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','ATM', TO_DATE('29/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','LEV', TO_DATE('29/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','MAL', TO_DATE('29/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','SEV', TO_DATE('29/09/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','RAY', TO_DATE('29/09/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 8 Sb. 5 y Dom. 6 Octubre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','BET', TO_DATE('05/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','CEL', TO_DATE('05/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','OSA', TO_DATE('05/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','RMA', TO_DATE('05/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','VAL', TO_DATE('05/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','GRA', TO_DATE('06/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','ESP', TO_DATE('06/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','VLL', TO_DATE('06/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','ALM', TO_DATE('06/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','RSO', TO_DATE('06/10/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 9 Sb. 19 y Dom. 20 Octubre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','VIL', TO_DATE('19/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','GET', TO_DATE('19/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','ELC', TO_DATE('19/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','ATM', TO_DATE('19/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','RAY', TO_DATE('19/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','LEV', TO_DATE('20/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','MAL', TO_DATE('20/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','FCB', TO_DATE('20/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','SEV', TO_DATE('20/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','RSO', TO_DATE('20/10/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 10 Sb. 26 y Dom. 27 Octubre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','VAL', TO_DATE('26/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','ATH', TO_DATE('26/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','GRA', TO_DATE('26/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','ALM', TO_DATE('26/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','RMA', TO_DATE('26/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','BET', TO_DATE('27/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','ESP', TO_DATE('27/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','CEL', TO_DATE('27/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','OSA', TO_DATE('27/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','VLL', TO_DATE('27/10/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 11 Mar. 29 y Mier. 30 Octubre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','ELC', TO_DATE('29/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','LEV', TO_DATE('29/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','MAL', TO_DATE('29/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','RSO', TO_DATE('29/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','GET', TO_DATE('29/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','ATM', TO_DATE('30/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','FCB', TO_DATE('30/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','SEV', TO_DATE('30/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','RAY', TO_DATE('30/10/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','ALM', TO_DATE('30/10/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 12 Sb. 2 y Dom. 3 Noviembre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','VAL', TO_DATE('02/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','BET', TO_DATE('02/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','OSA', TO_DATE('02/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','VLL', TO_DATE('02/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','VIL', TO_DATE('02/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','ATH', TO_DATE('03/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','GRA', TO_DATE('03/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','ESP', TO_DATE('03/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','CEL', TO_DATE('03/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','RMA', TO_DATE('03/11/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 13 Sb. 9 y Dom. 10 Noviembre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','MAL', TO_DATE('09/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','RSO', TO_DATE('09/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','ALM', TO_DATE('09/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','VLL', TO_DATE('09/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','ELC', TO_DATE('09/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','ATM', TO_DATE('10/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','LEV', TO_DATE('10/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','FCB', TO_DATE('10/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','RAY', TO_DATE('10/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','SEV', TO_DATE('10/11/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 14 Sb. 23 y Dom. 24 Noviembre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','VAL', TO_DATE('23/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','ATH', TO_DATE('23/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','GRA', TO_DATE('23/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','RMA', TO_DATE('23/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','GET', TO_DATE('23/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','VIL', TO_DATE('24/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','BET', TO_DATE('24/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','ESP', TO_DATE('24/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','CEL', TO_DATE('24/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','OSA', TO_DATE('24/11/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 15 Sb. 30 y Dom. 1 Diciembre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','ATM', TO_DATE('30/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','SEV', TO_DATE('30/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','RSO', TO_DATE('30/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','VLL', TO_DATE('30/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','OSA', TO_DATE('30/11/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','LEV', TO_DATE('01/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','MAL', TO_DATE('01/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','FCB', TO_DATE('01/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','RAY', TO_DATE('01/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','ALM', TO_DATE('01/12/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 16 Sb. 14 y Dom. 15 Diciembre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','ELC', TO_DATE('14/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','VAL', TO_DATE('14/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','GET', TO_DATE('14/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','VIL', TO_DATE('14/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','ATH', TO_DATE('14/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','GRA', TO_DATE('15/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','BET', TO_DATE('15/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','ESP', TO_DATE('15/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','CEL', TO_DATE('15/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','RMA', TO_DATE('15/12/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 17 Sb. 21 y Sb. 21 Diciembre
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','LEV', TO_DATE('21/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','MAL', TO_DATE('21/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','FCB', TO_DATE('21/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','SEV', TO_DATE('21/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','RAY', TO_DATE('21/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','RSO', TO_DATE('21/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','ALM', TO_DATE('21/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','VLL', TO_DATE('21/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','OSA', TO_DATE('21/12/2013', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','RMA', TO_DATE('21/12/2013', 'DD/MM/YYYY'), '1');

-- JORNADA 18 Sb. 4 y Dom. 5 Enero
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','LEV', TO_DATE('04/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','ATM', TO_DATE('04/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','ELC', TO_DATE('04/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','GET', TO_DATE('04/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','VIL', TO_DATE('04/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','ATH', TO_DATE('05/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','GRA', TO_DATE('05/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','BET', TO_DATE('05/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','ESP', TO_DATE('05/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','CEL', TO_DATE('05/01/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 19 Sb. 11 y Dom. 12 Enero
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','FCB', TO_DATE('11/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','RMA', TO_DATE('11/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','RAY', TO_DATE('11/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','OSA', TO_DATE('11/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','ALM', TO_DATE('11/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','SEV', TO_DATE('12/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','RSO', TO_DATE('12/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','VAL', TO_DATE('12/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','MAL', TO_DATE('12/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','VLL', TO_DATE('12/01/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 20 Sb. 18 y Dom. 19 Enero
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','FCB', TO_DATE('18/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','VAL', TO_DATE('18/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','RMA', TO_DATE('18/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','SEV', TO_DATE('18/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','RSO', TO_DATE('18/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','VLL', TO_DATE('19/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','OSA', TO_DATE('19/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','CEL', TO_DATE('19/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','ALM', TO_DATE('19/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','RAY', TO_DATE('19/01/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 21 Sb. 25 y Dom. 26 Enero
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','ESP', TO_DATE('25/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','BET', TO_DATE('25/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','GRA', TO_DATE('25/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','ATH', TO_DATE('25/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','VIL', TO_DATE('25/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','GET', TO_DATE('26/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','ELC', TO_DATE('26/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','ATM', TO_DATE('26/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','LEV', TO_DATE('26/01/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','MAL', TO_DATE('26/01/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 22 Sb. 1 y Dom. 2 Febrero
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','RMA', TO_DATE('01/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','VLL', TO_DATE('01/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','ALM', TO_DATE('01/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','RSO', TO_DATE('01/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','RAY', TO_DATE('01/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','SEV', TO_DATE('02/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','VAL', TO_DATE('02/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','ESP', TO_DATE('02/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','CEL', TO_DATE('02/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','OSA', TO_DATE('02/02/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 23 Sb. 8 y Dom. 9 Febrero
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','BET', TO_DATE('08/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','GRA', TO_DATE('08/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','ATH', TO_DATE('08/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','VIL', TO_DATE('08/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','GET', TO_DATE('08/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','ELC', TO_DATE('09/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','ATM', TO_DATE('09/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','LEV', TO_DATE('09/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','FCB', TO_DATE('09/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','MAL', TO_DATE('09/02/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 24 Sb. 15 y Dom. 16 Febrero
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','ESP', TO_DATE('15/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','CEL', TO_DATE('15/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','RMA', TO_DATE('15/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','OSA', TO_DATE('15/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','BET', TO_DATE('15/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','VLL', TO_DATE('16/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','ALM', TO_DATE('16/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','RSO', TO_DATE('16/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','RAY', TO_DATE('16/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','VAL', TO_DATE('16/02/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 25 Sb. 22 y Dom. 23 Febrero
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','GET', TO_DATE('22/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','MAL', TO_DATE('22/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','FCB', TO_DATE('22/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','ATM', TO_DATE('22/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','GRA', TO_DATE('22/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','ATH', TO_DATE('23/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','VIL', TO_DATE('23/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','ELC', TO_DATE('23/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','LEV', TO_DATE('23/02/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','SEV', TO_DATE('23/02/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 26 Sb. 1 y Lun. 3 Marzo
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','CEL', TO_DATE('01/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','GRA', TO_DATE('01/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','ALM', TO_DATE('01/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','BET', TO_DATE('01/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','ESP', TO_DATE('01/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','RMA', TO_DATE('01/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','OSA', TO_DATE('01/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','VLL', TO_DATE('03/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','RSO', TO_DATE('03/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','VAL', TO_DATE('03/03/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 27 Sb. 8 y Lun. 10 Marzo
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','GET', TO_DATE('08/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','ATM', TO_DATE('08/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','MAL', TO_DATE('08/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','LEV', TO_DATE('08/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','ATH', TO_DATE('08/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','VIL', TO_DATE('08/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','ELC', TO_DATE('08/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','FCB', TO_DATE('10/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','SEV', TO_DATE('10/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','RAY', TO_DATE('10/03/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 28 Sb. 15 y Lun. 17 Marzo
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','ATH', TO_DATE('15/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','GRA', TO_DATE('15/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','BET', TO_DATE('15/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','ESP', TO_DATE('15/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','ALM', TO_DATE('15/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','CEL', TO_DATE('15/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','RMA', TO_DATE('15/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','OSA', TO_DATE('17/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','VLL', TO_DATE('17/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','VAL', TO_DATE('17/03/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 29 Sb. 22 y Dom. 23 Marzo
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','VIL', TO_DATE('22/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','GET', TO_DATE('22/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','ELC', TO_DATE('22/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','RSO', TO_DATE('22/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','FCB', TO_DATE('22/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','ATM', TO_DATE('23/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','LEV', TO_DATE('23/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','MAL', TO_DATE('23/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','SEV', TO_DATE('23/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','RAY', TO_DATE('23/03/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 30 Mar. 25 y Jue. 27 Marzo
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','ATH', TO_DATE('25/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','BET', TO_DATE('25/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','ESP', TO_DATE('25/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','VLL', TO_DATE('25/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','VIL', TO_DATE('25/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','GRA', TO_DATE('27/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','CEL', TO_DATE('27/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','RMA', TO_DATE('27/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','OSA', TO_DATE('27/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','VAL', TO_DATE('27/03/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 31 Sb. 29 y Dom. 30 Marzo
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','GET', TO_DATE('29/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','MAL', TO_DATE('29/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','RSO', TO_DATE('29/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','ALM', TO_DATE('29/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','ELC', TO_DATE('29/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','ATM', TO_DATE('30/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','LEV', TO_DATE('30/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','FCB', TO_DATE('30/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','SEV', TO_DATE('30/03/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','RAY', TO_DATE('30/03/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 32 Sb. 5 y Dom. 6 Abril
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','GRA', TO_DATE('05/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','RMA', TO_DATE('05/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','OSA', TO_DATE('05/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','VAL', TO_DATE('05/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','GET', TO_DATE('05/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','VIL', TO_DATE('06/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','ATH', TO_DATE('06/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','BET', TO_DATE('06/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','CEL', TO_DATE('06/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','ESP', TO_DATE('06/04/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 33 Sb. 12 y Dom. 13 Abril
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','ELC', TO_DATE('12/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','MAL', TO_DATE('12/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','FCB', TO_DATE('12/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','ALM', TO_DATE('12/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','ATM', TO_DATE('12/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','LEV', TO_DATE('13/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','SEV', TO_DATE('13/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','RAY', TO_DATE('13/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','RSO', TO_DATE('13/04/2014', 'DD/MM/YYYY'), '1');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','VLL', TO_DATE('13/04/2014', 'DD/MM/YYYY'), '1');

-- JORNADA 34 Sb. 19 y Lun. 21 Abril
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','ELC', TO_DATE('18/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','GRA', TO_DATE('20/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','ESP', TO_DATE('19/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','RMA', TO_DATE('18/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','VAL', TO_DATE('19/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','GET', TO_DATE('19/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','VIL', TO_DATE('21/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','ATH', TO_DATE('20/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','BET', TO_DATE('20/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','CEL', TO_DATE('20/04/2014', 'DD/MM/YYYY'), '0');

-- JORNADA 35 Sb. 26 y Dom. 27 Abril
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','LEV', TO_DATE('25/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','ATM', TO_DATE('27/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','MAL', TO_DATE('26/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','FCB', TO_DATE('27/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','SEV', TO_DATE('27/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','RAY', TO_DATE('26/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','RSO', TO_DATE('26/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','ALM', TO_DATE('27/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','VLL', TO_DATE('28/04/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','OSA', TO_DATE('26/04/2014', 'DD/MM/YYYY'), '0');

-- JORNADA 36 Sb. 3 y Dom. 4 Mayo
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','ATM', TO_DATE('04/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','ELC', TO_DATE('03/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','GET', TO_DATE('03/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','VIL', TO_DATE('04/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','ATH', TO_DATE('02/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','GRA', TO_DATE('05/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','BET', TO_DATE('04/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','ESP', TO_DATE('03/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','CEL', TO_DATE('03/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','VAL', TO_DATE('04/05/2014', 'DD/MM/YYYY'), '0');

-- JORNADA 37 Sb. 10 y Dom. 11 Mayo
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'LEV','VAL', TO_DATE('10/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATM','MAL', TO_DATE('10/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ELC','FCB', TO_DATE('10/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GET','SEV', TO_DATE('10/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VIL','RAY', TO_DATE('10/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ATH','RSO', TO_DATE('11/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'GRA','ALM', TO_DATE('11/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'BET','VLL', TO_DATE('11/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ESP','OSA', TO_DATE('11/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'CEL','RMA', TO_DATE('11/05/2014', 'DD/MM/YYYY'), '0');

-- JORNADA 38 Sb. 17 y Dom. 18 Mayo
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'FCB','ATM', TO_DATE('17/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RMA','ESP', TO_DATE('17/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RAY','GET', TO_DATE('17/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'OSA','BET', TO_DATE('17/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'ALM','ATH', TO_DATE('17/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'SEV','ELC', TO_DATE('18/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'RSO','VIL', TO_DATE('18/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VAL','CEL', TO_DATE('18/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'MAL','LEV', TO_DATE('18/05/2014', 'DD/MM/YYYY'), '0');
      RETORN := RETORN + PKG_PARTITS.ALTA_PARTIT('BBVA', 2013, 'VLL','GRA', TO_DATE('18/05/2014', 'DD/MM/YYYY'), '0');
      
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_PARTITS');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_PARTITS;

  FUNCTION DADES_INICIALS_TIPUS_GOL RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      RETORN := RETORN + PKG_PARTITS.ALTA_TIPUS_GOL ('P', 'Penal');
      RETORN := RETORN + PKG_PARTITS.ALTA_TIPUS_GOL ('F', 'Falta');
      RETORN := RETORN + PKG_PARTITS.ALTA_TIPUS_GOL ('C', 'Corner');
      RETORN := RETORN + PKG_PARTITS.ALTA_TIPUS_GOL ('X', 'Xut');
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_TIPUS_GOL');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_TIPUS_GOL;

  FUNCTION DADES_INICIALS_GOLS RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      RETORN := RETORN + PKG_PARTITS.ALTA_GOL( PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), 'FCB', PKG_COMPETICIONS.CERCA_JUGADOR ('Lionel', 'Messi', ''),      1, 35, 'X');
      RETORN := RETORN + PKG_PARTITS.ALTA_GOL( PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), 'RMA', PKG_COMPETICIONS.CERCA_JUGADOR ('Cristiano', 'Ronaldo', ''), 1, 40, 'P');
      RETORN := RETORN + PKG_PARTITS.ALTA_GOL( PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), 'FCB', PKG_COMPETICIONS.CERCA_JUGADOR ('Lionel', 'Messi', ''),      2, 67, 'F');
      RETORN := RETORN + PKG_PARTITS.ALTA_GOL( PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), 'FCB', PKG_COMPETICIONS.CERCA_JUGADOR ('Lionel', 'Messi', ''),      2, 74, 'C');

      RETORN := RETORN + PKG_PARTITS.ALTA_GOL( PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'RMA', 'FCB'), 'FCB', PKG_COMPETICIONS.CERCA_JUGADOR ('Lionel', 'Messi', ''),      1, 15, 'X');
      RETORN := RETORN + PKG_PARTITS.ALTA_GOL( PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'RMA', 'FCB'), 'FCB', PKG_COMPETICIONS.CERCA_JUGADOR ('Lionel', 'Messi', ''),      1, 25, 'F');
      RETORN := RETORN + PKG_PARTITS.ALTA_GOL( PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'RMA', 'FCB'), 'FCB', PKG_COMPETICIONS.CERCA_JUGADOR ('Lionel', 'Messi', ''),      2, 80, 'C');
      RETORN := RETORN + PKG_PARTITS.ALTA_GOL( PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'RMA', 'FCB'), 'RMA', PKG_COMPETICIONS.CERCA_JUGADOR ('Cristiano', 'Ronaldo', ''), 2, 91, 'X');
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_GOLS');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_GOLS;

  FUNCTION DADES_INICIALS_USUARIS RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      RETORN := RETORN + PKG_USUARIS.ALTA_USUARI('CASTILLAMJ@UOC.COM', 'UOC-PASS', 'Jordi', 'Castilla', 'Mir', 'H', 'D', '37332606H', TO_DATE('19/11/1968', 'DD/MM/YYYY'), 'CAT', 'CAT', 'ES5621000591220200154578', '4518081367205539');
      RETORN := RETORN + PKG_USUARIS.ALTA_USUARI('DONA-1970@UOC.COM', 'UOC-PASS', 'Maria', 'Cognom', '', 'D', 'D', '11111111A', TO_DATE('01/02/1970', 'DD/MM/YYYY'), 'CAT', 'CAT', 'ES56 2100 ....', '....');
      RETORN := RETORN + PKG_USUARIS.ALTA_USUARI('HOME-1990@UOC.COM', 'UOC-PASS', 'Joan', 'Cognom', '', 'H', 'D', '22222222B', TO_DATE('03/04/1990', 'DD/MM/YYYY'), 'CAT', 'CAT', 'ES56 2100 ....', '....');
      RETORN := RETORN + PKG_USUARIS.ALTA_USUARI('DONA-1993@UOC.COM', 'UOC-PASS', 'Montserrat', 'Cognom', '', 'D', 'D', '33333333C', TO_DATE('05/06/1993', 'DD/MM/YYYY'), 'CAT', 'CAT', 'ES56 2100 ....', '....');
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_USUARIS');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_USUARIS;

  FUNCTION DADES_INICIALS_MODALITATS RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      RETORN := RETORN + PKG_APOSTES.ALTA_MODALITAT('1X2');
      RETORN := RETORN + PKG_APOSTES.ALTA_MODALITAT('RESULTAT FINAL');
      RETORN := RETORN + PKG_APOSTES.ALTA_MODALITAT('EQUIP QUE MARCARA PRIMER');
      RETORN := RETORN + PKG_APOSTES.ALTA_MODALITAT('JUGADOR QUE MARCARA PRIMER');
      RETORN := RETORN + PKG_APOSTES.ALTA_MODALITAT('MINUT DEL PRIMER GOL');
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_MODALITATS');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_MODALITATS;

  FUNCTION DADES_INICIALS_COEFICIENTS RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1', 1.75 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), 'X', 1.95 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '2', 2.15 );

      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '0-0', 1.95 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1-0', 1.75 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '2-0', 1.85 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '3-0', 2.25 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '4-0', 3.10 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '0-1', 1.85 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1-1', 1.95 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '2-1', 2.25 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '3-1', 2.55 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '4-1', 2.85 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '0-2', 1.95 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1-2', 2.05 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '2-2', 2.45 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '3-2', 2.95 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '4-2', 3.35 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '0-3', 2.15 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1-3', 2.35 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '2-3', 2.75 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '3-3', 3.45 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '4-3', 3.95 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '0-4', 2.20 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1-4', 2.50 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '2-4', 2.90 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '3-4', 3.75 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '4-4', 4.25 );

      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('EQUIP QUE MARCARA PRIMER'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), 'FCB', 1.75 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('EQUIP QUE MARCARA PRIMER'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), 'RMA', 2.15 );

      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('JUGADOR QUE MARCARA PRIMER'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), PKG_COMPETICIONS.CERCA_JUGADOR ('Lionel', 'Messi', ''), 1.65 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('JUGADOR QUE MARCARA PRIMER'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), PKG_COMPETICIONS.CERCA_JUGADOR ('Cristiano', 'Ronaldo', ''), 2.35 );

      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MINUT DEL PRIMER GOL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '0-9', 3.85 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MINUT DEL PRIMER GOL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '10-19', 3.15 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MINUT DEL PRIMER GOL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '20-29', 2.45 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MINUT DEL PRIMER GOL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '30-39', 2.55 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MINUT DEL PRIMER GOL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '40-49', 1.95 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MINUT DEL PRIMER GOL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '50-59', 2.15 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MINUT DEL PRIMER GOL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '60-69', 2.00 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MINUT DEL PRIMER GOL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '70-79', 1.45 );
      RETORN := RETORN + PKG_APOSTES.ALTA_COEFICIENT( PKG_APOSTES.CERCA_MODALITAT('MINUT DEL PRIMER GOL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '80-90', 4.15 );

      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_COEFICIENTS');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_COEFICIENTS;

  FUNCTION DADES_INICIALS_APOSTES RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      RETORN := RETORN + PKG_JOC.LOGIN('CASTILLAMJ@UOC.COM', 'UOC-PASS');
      RETORN := RETORN + PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1', 15.00 );
      RETORN := RETORN + PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '4-0', 12.50 );
      
      RETORN := RETORN + PKG_JOC.LOGIN('DONA-1970@UOC.COM', 'UOC-PASS');
      RETORN := RETORN + PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1', 5.00 );
      RETORN := RETORN + PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '2-1', 5.00 );
      
      RETORN := RETORN + PKG_JOC.LOGIN('HOME-1990@UOC.COM', 'UOC-PASS');
      RETORN := RETORN + PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '2', 25.00 );
      RETORN := RETORN + PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1-3', 25.00 );
      
      RETORN := RETORN + PKG_JOC.LOGIN('DONA-1993@UOC.COM', 'UOC-PASS');
      RETORN := RETORN + PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1', 3.00 );
      RETORN := RETORN + PKG_APOSTES.ALTA_APOSTA( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '3-2', 1.50 );
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_APOSTES');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_APOSTES;

  FUNCTION DADES_INICIALS_RESULTATS RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      RETORN := 0;
      --RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'),                        PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '1');
      --RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'),             PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '3-0');
      --RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('EQUIP QUE MARCARA PRIMER'),   PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), 'FCB');
      --RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('JUGADOR QUE MARCARA PRIMER'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), PKG_COMPETICIONS.CERCA_JUGADOR('Lionel', 'Messi', ''));
      --RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('MINUT DEL PRIMER GOL'),       PKG_PARTITS.CERCA_PARTIT('BBVA', 2013, 'FCB', 'RMA'), '23');
--
-- Resultats 1X2
--
-- JORNADA 1 Sb. 17 y Dom. 18 Agosto
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','LEV'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','MAL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','BET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','ATM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','GET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','ATH'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','GRA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','ESP'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','VIL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','ELC'), '1');

-- JORNADA 2 Sb. 24 y Dom. 25 Agosto
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','VAL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','CEL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','RMA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','OSA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','VLL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','ALM'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','RSO'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','RAY'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','SEV'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','FCB'), '2');

-- JORNADA 3 Sb. 31 y Dom. 1 Septiembre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','ATH'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','GET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','ELC'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','ATM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','LEV'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','MAL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','FCB'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','BET'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','GRA'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','VIL'), '2');

-- JORNADA 4 Sb. 14 y Dom. 15 Septiembre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','VAL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','ESP'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','CEL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','RMA'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','OSA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','VLL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','ALM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','RSO'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','SEV'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','RAY'), '1');

-- JORNADA 5 Sb. 21 y Dom. 22 Septiembre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','ATH'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','VIL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','GET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','ELC'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','GRA'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','ATM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','LEV'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','MAL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','FCB'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','SEV'), '1');

-- JORNADA 6 Mar. 24 y Mier. 25 Septiembre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','CEL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','ALM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','RSO'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','OSA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','VAL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','BET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','ESP'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','RMA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','VLL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','RAY'), '1');

-- JORNADA 7 Sb. 28 y Dom. 29 Septiembre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','ELC'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','ATH'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','FCB'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','VIL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','GET'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','ATM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','LEV'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','MAL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','SEV'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','RAY'), '1');

-- JORNADA 8 Sb. 5 y Dom. 6 Octubre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','BET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','CEL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','OSA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','RMA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','VAL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','GRA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','ESP'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','VLL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','ALM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','RSO'), '1');

-- JORNADA 9 Sb. 19 y Dom. 20 Octubre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','VIL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','GET'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','ELC'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','ATM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','RAY'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','LEV'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','MAL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','FCB'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','SEV'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','RSO'), '2');

-- JORNADA 10 Sb. 26 y Dom. 27 Octubre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','VAL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','ATH'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','GRA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','ALM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','RMA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','BET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','ESP'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','CEL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','OSA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','VLL'), '2');

-- JORNADA 11 Mar. 29 y Mier. 30 Octubre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','ELC'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','LEV'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','MAL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','RSO'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','GET'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','ATM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','FCB'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','SEV'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','RAY'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','ALM'), '2');

-- JORNADA 12 Sb. 2 y Dom. 3 Noviembre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','VAL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','BET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','OSA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','VLL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','VIL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','ATH'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','GRA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','ESP'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','CEL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','RMA'), '2');

-- JORNADA 13 Sb. 9 y Dom. 10 Noviembre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','MAL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','RSO'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','ALM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','VLL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','ELC'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','ATM'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','LEV'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','FCB'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','RAY'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','SEV'), '2');

-- JORNADA 14 Sb. 23 y Dom. 24 Noviembre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','VAL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','ATH'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','GRA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','RMA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','GET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','VIL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','BET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','ESP'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','CEL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','OSA'), '2');

-- JORNADA 15 Sb. 30 y Dom. 1 Diciembre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','ATM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','SEV'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','RSO'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','VLL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','OSA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','LEV'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','MAL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','FCB'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','RAY'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','ALM'), '1');

-- JORNADA 16 Sb. 14 y Dom. 15 Diciembre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','ELC'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','VAL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','GET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','VIL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','ATH'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','GRA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','BET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','ESP'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','CEL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','RMA'), 'X');

-- JORNADA 17 Sb. 21 y Sb. 21 Diciembre
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','LEV'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','MAL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','FCB'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','SEV'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','RAY'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','RSO'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','ALM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','VLL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','OSA'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','RMA'), '2');

-- JORNADA 18 Sb. 4 y Dom. 5 Enero
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','LEV'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','ATM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','ELC'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','GET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','VIL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','ATH'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','GRA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','BET'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','ESP'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','CEL'), '1');

-- JORNADA 19 Sb. 11 y Dom. 12 Enero
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','FCB'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','RMA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','RAY'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','OSA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','ALM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','SEV'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','RSO'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','VAL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','MAL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','VLL'), '1');

-- JORNADA 20 Sb. 18 y Dom. 19 Enero
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','FCB'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','VAL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','RMA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','SEV'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','RSO'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','VLL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','OSA'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','CEL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','ALM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','RAY'), '1');

-- JORNADA 21 Sb. 25 y Dom. 26 Enero
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','ESP'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','BET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','GRA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','ATH'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','VIL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','GET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','ELC'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','ATM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','LEV'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','MAL'), '1');

-- JORNADA 22 Sb. 1 y Dom. 2 Febrero
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','RMA'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','VLL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','ALM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','RSO'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','RAY'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','SEV'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','VAL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','ESP'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','CEL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','OSA'), '1');

-- JORNADA 23 Sb. 8 y Dom. 9 Febrero
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','BET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','GRA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','ATH'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','VIL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','GET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','ELC'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','ATM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','LEV'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','FCB'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','MAL'), '1');

-- JORNADA 24 Sb. 15 y Dom. 16 Febrero
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','ESP'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','CEL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','RMA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','OSA'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','BET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','VLL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','ALM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','RSO'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','RAY'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','VAL'), 'X');

-- JORNADA 25 Sb. 22 y Dom. 23 Febrero
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','GET'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','MAL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','FCB'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','ATM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','GRA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','ATH'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','VIL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','ELC'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','LEV'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','SEV'), '2');

-- JORNADA 26 Sb. 1 y Lun. 3 Marzo
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','CEL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','GRA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','ALM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','BET'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','ESP'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','RMA'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','OSA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','VLL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','RSO'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','VAL'), '1');

-- JORNADA 27 Sb. 8 y Lun. 10 Marzo
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','GET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','ATM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','MAL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','LEV'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','ATH'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','VIL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','ELC'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','FCB'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','SEV'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','RAY'), '2');

-- JORNADA 28 Sb. 15 y Lun. 17 Marzo
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','ATH'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','GRA'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','BET'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','ESP'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','ALM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','CEL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','RMA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','OSA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','VLL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','VAL'), '1');

-- JORNADA 29 Sb. 22 y Dom. 23 Marzo
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','VIL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','GET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','ELC'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','RSO'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','FCB'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','ATM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','LEV'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','MAL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','SEV'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','RAY'), 'X');

-- JORNADA 30 Mar. 25 y Jue. 27 Marzo
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','ATH'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','BET'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','ESP'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','VLL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','VIL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','GRA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','CEL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','RMA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','OSA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','VAL'), 'X');

-- JORNADA 31 Sb. 29 y Dom. 30 Marzo
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','GET'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','MAL'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','RSO'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','ALM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','ELC'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','ATM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','LEV'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','FCB'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','SEV'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','RAY'), '1');

-- JORNADA 32 Sb. 5 y Dom. 6 Abril
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','GRA'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','RMA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','OSA'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','VAL'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','GET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','VIL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','ATH'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','BET'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','CEL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','ESP'), '1');

-- JORNADA 33 Sb. 12 y Dom. 13 Abril
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','ELC'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','MAL'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','FCB'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','ALM'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','ATM'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','LEV'), '1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','SEV'), '2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','RAY'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','RSO'), 'X');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('1X2'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','VLL'), 'X');

-- JORNADA 34 Sb. 19 y Lun. 21 Abril

-- JORNADA 35 Sb. 26 y Dom. 27 Abril

-- JORNADA 36 Sb. 3 y Dom. 4 Mayo

-- JORNADA 37 Sb. 10 y Dom. 11 Mayo

-- JORNADA 38 Sb. 17 y Dom. 18 Mayo


--
-- Resultats finals
--
-- JORNADA 1 Sb. 17 y Dom. 18 Agosto  
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','LEV'), '7-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','MAL'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','BET'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','ATM'), '1-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','GET'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','ATH'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','GRA'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','ESP'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','VIL'), '2-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','ELC'), '3-0');

-- JORNADA 2 Sb. 24 y Dom. 25 Agosto   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','VAL'), '3-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','CEL'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','RMA'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','OSA'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','VLL'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','ALM'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','RSO'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','RAY'), '5-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','SEV'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','FCB'), '0-1');

-- JORNADA 3 Sb. 31 y Dom. 1 Septiembre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','ATH'), '3-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','GET'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','ELC'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','ATM'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','LEV'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','MAL'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','FCB'), '2-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','BET'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','GRA'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','VIL'), '0-3');

-- JORNADA 4 Sb. 14 y Dom. 15 Septiembre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','VAL'), '3-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','ESP'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','CEL'), '3-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','RMA'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','OSA'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','VLL'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','ALM'), '4-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','RSO'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','SEV'), '3-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','RAY'), '5-0');

-- JORNADA 5 Sb. 21 y Dom. 22 Septiembre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','ATH'), '3-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','VIL'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','GET'), '4-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','ELC'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','GRA'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','ATM'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','LEV'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','MAL'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','FCB'), '0-4');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','SEV'), '3-1');

-- JORNADA 6 Mar. 24 y Mier. 25 Septiembre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','CEL'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','ALM'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','RSO'), '4-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','OSA'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','VAL'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','BET'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','ESP'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','RMA'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','VLL'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','RAY'), '4-1');

-- JORNADA 7 Sb. 28 y Dom. 29 Septiembre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','ELC'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','ATH'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','FCB'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','VIL'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','GET'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','ATM'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','LEV'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','MAL'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','SEV'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','RAY'), '1-0');

-- JORNADA 8 Sb. 5 y Dom. 6 Octubre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','BET'), '3-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','CEL'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','OSA'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','RMA'), '2-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','VAL'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','GRA'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','ESP'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','VLL'), '4-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','ALM'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','RSO'), '1-0');

-- JORNADA 9 Sb. 19 y Dom. 20 Octubre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','VIL'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','GET'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','ELC'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','ATM'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','RAY'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','LEV'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','MAL'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','FCB'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','SEV'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','RSO'), '1-2');

-- JORNADA 10 Sb. 26 y Dom. 27 Octubre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','VAL'), '4-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','ATH'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','GRA'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','ALM'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','RMA'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','BET'), '5-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','ESP'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','CEL'), '0-5');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','OSA'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','VLL'), '0-3');

-- JORNADA 11 Mar. 29 y Mier. 30 Octubre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','ELC'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','LEV'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','MAL'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','RSO'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','GET'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','ATM'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','FCB'), '0-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','SEV'), '7-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','RAY'), '3-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','ALM'), '1-2');

-- JORNADA 12 Sb. 2 y Dom. 3 Noviembre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','VAL'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','BET'), '3-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','OSA'), '5-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','VLL'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','VIL'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','ATH'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','GRA'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','ESP'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','CEL'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','RMA'), '2-3');

-- JORNADA 13 Sb. 9 y Dom. 10 Noviembre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','MAL'), '3-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','RSO'), '5-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','ALM'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','VLL'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','ELC'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','ATM'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','LEV'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','FCB'), '1-4');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','RAY'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','SEV'), '1-3');

-- JORNADA 14 Sb. 23 y Dom. 24 Noviembre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','VAL'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','ATH'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','GRA'), '4-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','RMA'), '0-5');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','GET'), '7-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','VIL'), '0-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','BET'), '4-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','ESP'), '1-4');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','CEL'), '4-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','OSA'), '0-1');

-- JORNADA 15 Sb. 30 y Dom. 1 Diciembre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','ATM'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','SEV'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','RSO'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','VLL'), '4-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','OSA'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','LEV'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','MAL'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','FCB'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','RAY'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','ALM'), '3-1');

-- JORNADA 16 Sb. 14 y Dom. 15 Diciembre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','ELC'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','VAL'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','GET'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','VIL'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','ATH'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','GRA'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','BET'), '5-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','ESP'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','CEL'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','RMA'), '2-2');

-- JORNADA 17 Sb. 21 y Sb. 21 Diciembre   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','LEV'), '3-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','MAL'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','FCB'), '2-5');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','SEV'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','RAY'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','RSO'), '1-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','ALM'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','VLL'), '4-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','OSA'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','RMA'), '2-3');

-- JORNADA 18 Sb. 4 y Dom. 5 Enero   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','LEV'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','ATM'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','ELC'), '4-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','GET'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','VIL'), '2-5');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','ATH'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','GRA'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','BET'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','ESP'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','CEL'), '3-0');

-- JORNADA 19 Sb. 11 y Dom. 12 Enero   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','FCB'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','RMA'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','RAY'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','OSA'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','ALM'), '6-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','SEV'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','RSO'), '5-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','VAL'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','MAL'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','VLL'), '4-0');

-- JORNADA 20 Sb. 18 y Dom. 19 Enero   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','FCB'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','VAL'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','RMA'), '0-5');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','SEV'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','RSO'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','VLL'), '4-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','OSA'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','CEL'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','ALM'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','RAY'), '2-0');

-- JORNADA 21 Sb. 25 y Dom. 26 Enero   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','ESP'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','BET'), '4-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','GRA'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','ATH'), '1-5');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','VIL'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','GET'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','ELC'), '4-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','ATM'), '2-4');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','LEV'), '2-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','MAL'), '3-0');

-- JORNADA 22 Sb. 1 y Dom. 2 Febrero   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','RMA'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','VLL'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','ALM'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','RSO'), '4-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','RAY'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','SEV'), '3-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','VAL'), '2-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','ESP'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','CEL'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','OSA'), '3-1');

-- JORNADA 23 Sb. 8 y Dom. 9 Febrero   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','BET'), '5-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','GRA'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','ATH'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','VIL'), '4-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','GET'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','ELC'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','ATM'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','LEV'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','FCB'), '1-4');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','MAL'), '4-1');

-- JORNADA 24 Sb. 15 y Dom. 16 Febrero   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','ESP'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','CEL'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','RMA'), '0-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','OSA'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','BET'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','VLL'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','ALM'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','RSO'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','RAY'), '6-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','VAL'), '0-0');

-- JORNADA 25 Sb. 22 y Dom. 23 Febrero   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','GET'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','MAL'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','FCB'), '3-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','ATM'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','GRA'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','ATH'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','VIL'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','ELC'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','LEV'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','SEV'), '0-1');

-- JORNADA 26 Sb. 1 y Lun. 3 Marzo   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','CEL'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','GRA'), '4-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','ALM'), '4-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','BET'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','ESP'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','RMA'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','OSA'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','VLL'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','RSO'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','VAL'), '1-0');

-- JORNADA 27 Sb. 8 y Lun. 10 Marzo   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','GET'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','ATM'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','MAL'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','LEV'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','ATH'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','VIL'), '2-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','ELC'), '3-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','FCB'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','SEV'), '1-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','RAY'), '2-3');

-- JORNADA 28 Sb. 15 y Lun. 17 Marzo   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','ATH'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','GRA'), '3-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','BET'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','ESP'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','ALM'), '3-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','CEL'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','RMA'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','OSA'), '7-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','VLL'), '4-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','VAL'), '1-0');

-- JORNADA 29 Sb. 22 y Dom. 23 Marzo   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','VIL'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','GET'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','ELC'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','RSO'), '4-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','FCB'), '3-4');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','ATM'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','LEV'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','MAL'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','SEV'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','RAY'), '1-1');

-- JORNADA 30 Mar. 25 y Jue. 27 Marzo   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','ATH'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','BET'), '1-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','ESP'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','VLL'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','VIL'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','GRA'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','CEL'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','RMA'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','OSA'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','VAL'), '2-2');

-- JORNADA 31 Sb. 29 y Dom. 30 Marzo   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','GET'), '1-3');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','MAL'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','RSO'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','ALM'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','ELC'), '1-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','ATM'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','LEV'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','FCB'), '0-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','SEV'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','RAY'), '5-0');

-- JORNADA 32 Sb. 5 y Dom. 6 Abril   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','GRA'), '4-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','RMA'), '0-4');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','OSA'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','VAL'), '0-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','GET'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','VIL'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','ATH'), '1-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','BET'), '3-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','CEL'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','ESP'), '4-1');

-- JORNADA 33 Sb. 12 y Dom. 13 Abril   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','ELC'), '2-1');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','MAL'), '3-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','FCB'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','ALM'), '4-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','ATM'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','LEV'), '1-0');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','SEV'), '0-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','RAY'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','RSO'), '2-2');
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','VLL'), '0-0');

/*
-- JORNADA 34 Sb. 19 y Lun. 21 Abril   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','ELC'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','GRA'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','ESP'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','RMA'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','VAL'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','GET'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','VIL'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','ATH'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','BET'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','CEL'), ''); 

-- JORNADA 35 Sb. 26 y Dom. 27 Abril   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','LEV'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','ATM'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','MAL'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','FCB'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','SEV'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','RAY'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','RSO'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','ALM'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','VLL'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','OSA'), ''); 

-- JORNADA 36 Sb. 3 y Dom. 4 Mayo   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','ATM'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','ELC'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','GET'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','VIL'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','ATH'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','GRA'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','BET'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','ESP'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','CEL'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','VAL'), ''); 

-- JORNADA 37 Sb. 10 y Dom. 11 Mayo   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'LEV','VAL'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATM','MAL'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ELC','FCB'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GET','SEV'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VIL','RAY'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ATH','RSO'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'GRA','ALM'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'BET','VLL'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ESP','OSA'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'CEL','RMA'), ''); 

-- JORNADA 38 Sb. 17 y Dom. 18 Mayo   
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'FCB','ATM'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RMA','ESP'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RAY','GET'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'OSA','BET'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'ALM','ATH'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'SEV','ELC'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'RSO','VIL'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VAL','CEL'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'MAL','LEV'), ''); 
      RETORN := RETORN + PKG_APOSTES.ALTA_RESULTAT( PKG_APOSTES.CERCA_MODALITAT('RESULTAT FINAL'), PKG_PARTITS.CERCA_PARTIT('BBVA', 2013,'VLL','GRA'), ''); 
*/
      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: DADES_INICIALS_RESULTATS');
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS_RESULTATS;


-- ********************

  FUNCTION DADES_INICIALS RETURN NUMBER IS
  BEGIN
    DECLARE
      RETORN NUMBER;
    BEGIN
      --
      -- ZONES
      --
      RETORN := 0;
      RETORN := RETORN + DADES_INICIALS_ZONES;

      --
      -- COMPETICIONS
      --
      RETORN := RETORN + DADES_INICIALS_COMPETICIONS;
      
      --
      -- TEMPORADES
      --
      RETORN := RETORN + DADES_INICIALS_TEMPORADES;
      
      --
      -- EQUIPS
      --
      RETORN := RETORN + DADES_INICIALS_EQUIPS;
      
      --
      -- JUGADORS
      --
      RETORN := RETORN + DADES_INICIALS_JUGADORS;

      --
      -- PLANTILLES
      --
      RETORN := RETORN + DADES_INICIALS_PLANTILLES;

      --
      -- PARTITS
      --
      RETORN := RETORN + DADES_INICIALS_PARTITS;

      --
      -- TIPUS_GOL
      --
      RETORN := RETORN + DADES_INICIALS_TIPUS_GOL;

      --
      -- GOLS
      --
      RETORN := RETORN + DADES_INICIALS_GOLS;

      --
      -- USUARIS
      --
      RETORN := RETORN + DADES_INICIALS_USUARIS;

      --
      -- MODALITATS
      --
      RETORN := RETORN + DADES_INICIALS_MODALITATS;

      --
      -- COEFICIENTS
      --
      RETORN := RETORN + DADES_INICIALS_COEFICIENTS;

      --
      -- APOSTES
      --
      RETORN := RETORN + DADES_INICIALS_APOSTES;

      --
      -- RESULTATS
      --
      RETORN := RETORN + DADES_INICIALS_RESULTATS;

      IF (RETORN <> 0) THEN
        DBMS_OUTPUT.PUT_LINE('ERROR: en algun pas de DADES_INICIALS');
        ROLLBACK;
      ELSE
        DBMS_OUTPUT.PUT_LINE('INFO: DADES_INICIALS OK');
        COMMIT;    
      END IF;

      RETURN RETORN;
    END;
  END DADES_INICIALS;

END PKG_UTIL;
/
